// [ICH] This is a very specific shader that handles several group of transparent and glowing vertices of a single mesh
//       Can be used to draw sun, clouds, moon stars in a single draw call
//       Each bone controls a group of vertex
//       This shader uses premultiply alpha to handle normal blending and additif blending

#ifdef DCC_MAX
	#define DCC( x ) x 
#else
	#define DCC( x )
#endif

#define NUM_BONES 8
#define NUM_WEIGHT_MASK 1

#if !defined(DCC_MAX)
	uniform lowp vec4 BoneAlphaValues[NUM_BONES];
	uniform highp mat4 BoneMatrices[NUM_BONES];
	uniform highp vec4 WeightMask;
	attribute mediump vec4 SkinWeights;
	attribute mediump vec4 SkinIndices;
#endif

attribute highp vec4 Vertex DCC(: POSITION);
attribute mediump vec2 TexCoord0 DCC(: TEXCOORD0);

uniform highp mat4 WorldViewProjectionMatrix;

varying lowp vec4 vColor DCC(: COLOR0);
varying	mediump vec2 vCoord0 DCC(: TEXCOORD0);

void main(void)
{
#if !defined(DCC_MAX)
    highp vec4 weights = SkinWeights * WeightMask;
    highp vec4 netPosition = vec4(0.0, 0.0, 0.0, 0.0);
    vColor = vec4(0.0, 0.0, 0.0, 0.0);

	vec4 indices = SkinIndices;

    for (int i = 0; i < NUM_WEIGHT_MASK; i++)
    {
        int index = int(indices[i]);
        netPosition += weights[i] * (BoneMatrices[index] * vec4(Vertex.xyz, 1));        
        vColor += weights[i] * BoneAlphaValues[index];  
    }
  
 	gl_Position = WorldViewProjectionMatrix * netPosition;
#else
    vColor = vec4(1.0, 1.0, 1.0, 1.0);
	gl_Position = WorldViewProjectionMatrix * Vertex;
#endif

	vCoord0 = TexCoord0;
}
