uniform lowp sampler2D texture0;

varying mediump vec2 vCoord0;
varying mediump vec2 vCoordL1;
varying mediump vec2 vCoordL2;
varying mediump vec2 vCoordL3;
varying mediump vec2 vCoordL4;
varying mediump vec2 vCoordR1;
varying mediump vec2 vCoordR2;
varying mediump vec2 vCoordR3;
varying mediump vec2 vCoordR4;

//weight list per samples count based on Gaussian Distribution using sigma = 5, with sum normalized
//weights for 3:	0.338, 0.331
//weights for 5:	0.208, 0.204, 0.192
//weights for 7:	0.154, 0.151, 0.143, 0.129
//weights for 9:	0.16, 0.15, 0.12, 0.09, 0.05
//weights for 11:	0.110, 0.107, 0.101, 0.092, 0.08, 0.066

//we needed to pass to mediump for color calculation (which is almost twice as expensive than lowp...), 
//because the result is horrible on device in lowp.
//
//To limit the increase in cycle count we:
//- went down to 5 samples.
//- removed Guassian weight. (not a big issue at 5 samples, since all weights are really close to 0.2)
//In the end, this gives us +3 cycles...

void main()
{
	//get blurred color
	lowp vec4 blurred = vec4(0.0);
   	
   	blurred += (texture2D(texture0, vCoordL2) + texture2D(texture0, vCoordR2));
   	blurred += (texture2D(texture0, vCoordL1) + texture2D(texture0, vCoordR1));
   	blurred += texture2D(texture0, vCoord0);
   	
   	blurred *= 0.2;

	gl_FragColor = blurred;
}
