uniform lowp mat4 ColorMatrix; // Android

#ifdef DCC_MAX
	#define DCC(x) x 
#else
	#define DCC(x)
#endif

//#define USE_MIX_FOR_REFLECTION

//#define FG

#ifdef DCC_MAX
	#define USE_SPECULAR_LIGHTING
	#define USE_STATIC_REFLECTION
	#define USE_DYNAMIC_REFLECTION
#else
	//#define USE_SPECULAR_LIGHTING
	//#define USE_STATIC_REFLECTION	
	//#define USE_DYNAMIC_REFLECTION
	//#define DIFFUSE_ALPHA_REFLECTION
	//#define IS_GLASS_SHADER
	//#define COLOR_VAR
#endif

#ifdef IS_GLASS_SHADER
	#define StaticReflectionIntensity StaticGlassReflectionIntensity
	#define DynamicReflectionIntensity DynamicGlassReflectionIntensity
	#define ReflectionFalloffTexture GlassReflectionFalloffTexture
#endif

varying	mediump vec2 vCoord0 DCC(: TEXCOORD0);

varying lowp float vNDotL;
varying lowp float vNDotV;

#if defined(USE_SPECULAR_LIGHTING)
varying mediump vec2 specUV;

uniform lowp sampler2D CarSpecLUTSampler;
uniform lowp vec4 SpecularColor;
uniform lowp float CarSpecFactor;
#endif // USE_SPECULAR_LIGHTING

#if defined(USE_STATIC_REFLECTION)
varying	mediump vec3 vCoordCubeMap DCC(: TEXCOORD1);

uniform lowp samplerCube ReflectionSampler;
#endif

#if defined(USE_DYNAMIC_REFLECTION)
varying	mediump vec2 vCoordFrameBuffer DCC(: TEXCOORD2);

uniform lowp sampler2D FrameBuffer;
uniform lowp sampler2D FrameBuffer_alpha; // Android

uniform mediump float DynamicReflectionIntensity;
#endif

#if defined(USE_STATIC_REFLECTION) || defined(USE_DYNAMIC_REFLECTION)
varying	mediump	vec2 vCoordFalloff DCC(: TEXCOORD3);

uniform sampler2D ReflectionFalloffTexture;
uniform mediump float StaticReflectionIntensity; 
#endif // USE_DYNAMIC_REFLECTION || USE_STATIC_REFLECTION

uniform lowp sampler2D DiffuseSampler;
uniform lowp sampler2D DiffuseSampler_alpha; // Android

#ifdef ALPHA_TEST
	uniform lowp float AlphaRef;
#endif

#ifdef ALPHA_BLEND
	uniform lowp float Object_Alpha;
#else
	#define Object_Alpha 1.0
#endif

uniform lowp vec4 sunFactor;

uniform lowp vec4 AmbientColor;
uniform lowp vec4 DiffuseColor;
uniform lowp float SunOcclusionFactor;

uniform lowp vec3 destroyedColorOffset;
uniform lowp float destroyedReflectionFactor;

#ifdef FG
	varying lowp float vFogFactor;
	uniform lowp vec4 fogcolor;   
#endif

#ifdef COLOR_VAR
	uniform lowp sampler2D ColorVarMask;
	uniform lowp vec3 ColorVar1;
	uniform lowp vec3 ColorVar2;
	uniform lowp vec3 ColorVar3;
	uniform lowp vec3 ColorVar4;
#endif

#ifdef DCC_MAX
	#ifdef BYPASS_TONE_MAPPING
		#undef BYPASS_TONE_MAPPING
	#endif
#endif

#define TONED
#ifdef TONED
	uniform lowp vec3 ToneGain;
	uniform lowp vec3 ToneOffset;
#endif

void main()
{		
	//lowp vec4 color = texture2D(DiffuseSampler, vCoord0); // Android
	lowp vec4 color = vec4( texture2D(DiffuseSampler, vCoord0).rgb, texture2D(DiffuseSampler_alpha, vCoord0).g ) ; // Android

#ifdef ALPHA_TEST
	if (color.a < AlphaRef) discard;
#endif	

#ifdef COLOR_VAR
	lowp vec4 ColorMask = texture2D(ColorVarMask, vCoord0);
	
	lowp vec3 diffuseColor = color.rgb;
	diffuseColor += ColorVar1 * ColorMask.x;
	diffuseColor += ColorVar2 * ColorMask.y;
	diffuseColor += ColorVar3 * ColorMask.z;
	diffuseColor += ColorVar4 * ColorMask.w;
	color.rgb = min(diffuseColor, 1.0);
#endif

	color.rgb = max(color.rgb - destroyedColorOffset.rgb, 0.0);

	lowp vec3 light = AmbientColor.rgb + DiffuseColor.rgb * SunOcclusionFactor * vNDotL;

	#ifdef DCC_MAX
		if (light.rgb == vec3(0.0))
    		light = vec3(1.0);
	#endif	
	
	color.rgb *= light;
	
#if defined(USE_STATIC_REFLECTION) || defined(USE_DYNAMIC_REFLECTION)
	//lowp vec3 reflectionFalloffTex = texture2D(ReflectionFalloffTexture, vCoordFalloff).rgb; // Android
	lowp vec4 reflectionFalloffTex = texture2D(ReflectionFalloffTexture, vCoordFalloff); // Android
	
	color.rgb *= reflectionFalloffTex.b;
#endif

#if defined(USE_SPECULAR_LIGHTING)
	lowp float specPower = texture2D(CarSpecLUTSampler, specUV).a;
	lowp vec3 SpecularLight = SpecularColor.rgb * (SunOcclusionFactor * specPower * CarSpecFactor);
	
	#ifdef DIFFUSE_ALPHA_REFLECTION
		SpecularLight *= color.a;
	#endif
		
	color.rgb += SpecularLight;
#endif

	color.rgb += sunFactor.rgb;

#ifdef DCC_MAX
	light += vec3(1.0, 1.0, 1.0); //Overwrite light for max, without affecting bindings
#endif

#if defined(USE_STATIC_REFLECTION) || defined(USE_DYNAMIC_REFLECTION)

	mediump float reflectionMult = destroyedReflectionFactor * vNDotV; 

	#ifdef DIFFUSE_ALPHA_REFLECTION
		reflectionMult *= color.a;
	#endif

	#ifdef USE_DYNAMIC_REFLECTION
		//lowp vec3 frameBuffer = texture2D(FrameBuffer, vCoordFrameBuffer).rgb; // Android
		lowp vec4 frameBuffer = vec4( texture2D(FrameBuffer, vCoordFrameBuffer).rgb, texture2D(FrameBuffer_alpha, vCoordFrameBuffer).g ) ; // Android
		
		lowp float dynamicMult = DynamicReflectionIntensity * reflectionFalloffTex.r * reflectionMult;	
		#ifdef USE_MIX_FOR_REFLECTION
			color.rgb = mix(color.rgb, frameBuffer, dynamicMult);
		#else // USE_MIX_FOR_REFLECTION
			color.rgb += frameBuffer.rgb * dynamicMult;
		#endif // USE_MIX_FOR_REFLECTION
	#endif

	#ifdef USE_STATIC_REFLECTION
		lowp vec3 cubemapColor = textureCube(ReflectionSampler, vCoordCubeMap).rgb;
		lowp float staticMult = StaticReflectionIntensity * reflectionFalloffTex.g * reflectionMult;
		
		#ifdef USE_MIX_FOR_REFLECTION
			color.rgb = mix(color.rgb, cubemapColor, staticMult);
		#else // USE_MIX_FOR_REFLECTION
			color.rgb +=  cubemapColor.rgb * staticMult;
		#endif // USE_MIX_FOR_REFLECTION
	#endif
	
	#ifdef IS_GLASS_SHADER
		color.a = mix(color.a, 1.0, reflectionFalloffTex.g * vNDotV);
	#endif
#endif

#if !defined(IS_GLASS_SHADER)
	color.a = 1.0;
#endif

#ifdef ALPHA_BLEND
	color.a *= Object_Alpha;		
#endif

#ifdef FG
	color.xyz = mix(color.xyz, fogcolor.xyz, vFogFactor);
#endif

#if defined(TONED) && !defined(BYPASS_TONE_MAPPING)
	color.rgb = color.rgb * ToneGain + ToneOffset;
#endif

#ifdef DCC_MAX
	color.rgb = min(0, color.rgb) + texture2D(DiffuseSampler, vCoord0).rgb;
#endif

	gl_FragColor = color;
}