//#define FG

#ifdef DCC_MAX
	#define DCC( x ) x 
#else
	#define DCC( x )
#endif

#ifdef DCC_MAX
	#define USE_SPECULAR_LIGHTING
	#define USE_STATIC_REFLECTION
	#define USE_DYNAMIC_REFLECTION
#else
	//#define USE_SPECULAR_LIGHTING
	//#define USE_STATIC_REFLECTION
	//#define USE_DYNAMIC_REFLECTION
#endif

attribute highp vec4 Vertex DCC(: POSITION);
attribute mediump vec3 Normal DCC(: NORMAL);
attribute mediump vec2 TexCoord0 DCC(: TEXCOORD0);

varying	mediump vec2 vCoord0 DCC(: TEXCOORD0);

varying lowp float vNDotL;
varying lowp float vNDotV;

#if defined(USE_SPECULAR_LIGHTING)
varying mediump vec3 vViewDirection;
varying mediump vec2 specUV;
#endif // USE_SPECULAR_LIGHTING

#if defined(USE_STATIC_REFLECTION)
varying	mediump vec3 vCoordCubeMap DCC(: TEXCOORD1);
#endif

#if defined(USE_DYNAMIC_REFLECTION)
varying	mediump vec2 vCoordFrameBuffer DCC(: TEXCOORD2);

uniform mediump float FrameBufferScale;
#endif

#if defined(USE_STATIC_REFLECTION) || defined(USE_DYNAMIC_REFLECTION)
varying	mediump	vec2 vCoordFalloff DCC(: TEXCOORD3);
#endif // USE_DYNAMIC_REFLECTION || USE_STATIC_REFLECTION

uniform highp mat4 World;
uniform highp mat4 WorldIT;
uniform highp mat4 WorldViewProjection;
uniform	highp mat4 WorldViewIT;



uniform highp mat4 matWorld;

uniform lowp float SunOcclusionFactor;

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif


#if defined(SKINNED) && !defined(DCC_MAX)

#if !defined(BONE_COUNT)
#define BONE_COUNT 8
#endif

uniform highp mat4 BoneMatrices[BONE_COUNT];
uniform highp vec4 WeightMask;
attribute mediump vec4 SkinWeights;
attribute mediump vec4 SkinIndices;
#else
#endif //SKINNED

uniform mediump vec3 Light0DirectionOS;


#if defined(SKINNED) && !defined(DCC_MAX)
uniform mediump	vec4 EyePosition;
#else
uniform mediump	vec4 EyePositionOS;
#endif

void main(void)
{

#if defined SKINNED && !defined(DCC_MAX)
    highp vec4 weights = SkinWeights * WeightMask;
    highp vec4 netPosition = vec4(0.0, 0.0, 0.0, 0.0);
    highp vec4 netNormal = vec4(0.0, 0.0, 0.0, 0.0);

	vec4 indices = SkinIndices;

    for (int i = 0; i < 4; i++)
    {
        int index = int(indices[i]);
        netPosition += weights[i] * (BoneMatrices[index] * vec4(Vertex.xyz, 1));
        netNormal += weights[i] * (BoneMatrices[index] * vec4(Normal.xyz, 0));    
    }
 
  	gl_Position = WorldViewProjection * netPosition;
	vNDotL = max(0.0, dot(netNormal.xyz, Light0DirectionOS));	
	
	mediump vec3 vV = normalize(EyePosition.xyz - netPosition.xyz);
	
	vNDotV = step(0.0, dot(netNormal.xyz, vV));	
	
#if defined(USE_SPECULAR_LIGHTING) | defined(USE_STATIC_REFLECTION) || defined(USE_DYNAMIC_REFLECTION)

#ifdef USE_SPECULAR_LIGHTING
	mediump vec3 halfVector = normalize(Light0DirectionOS + vV);
	specUV = vec2(max(0.0, dot(netNormal.xyz, halfVector)), 0.0);
#endif

#if defined(USE_STATIC_REFLECTION) || defined(USE_DYNAMIC_REFLECTION)
	// calculate lighting
	lowp vec4 vR = vec4(reflect(-vV, netNormal.xyz), 0.0);
	vCoordFalloff = (WorldViewIT * vec4(netNormal.xyz, 0.0)).xy * vec2(0.49, -0.49) + vec2(0.5, 0.5);
#endif
	
#ifdef USE_STATIC_REFLECTION
	vCoordCubeMap = (World * vR).xyz;
#endif

#ifdef USE_DYNAMIC_REFLECTION
	//Compute the reflection vector
	vCoordFrameBuffer = vec2(0.46875, 0.3125) + vec2(0.46875, -0.3125) * (WorldViewIT * vec4(vR)).xy;
#endif 

#endif	
	
#else // SKINNED

	gl_Position = WorldViewProjection * Vertex;
	vNDotL = max(0.0, dot(Normal.xyz, Light0DirectionOS));

	highp vec3 netNormal = normalize(Normal.xyz);

	mediump vec3 vV = normalize(EyePositionOS.xyz - Vertex.xyz);
	
	vNDotV = step(0.0, dot(netNormal, vV));	

#if defined(USE_SPECULAR_LIGHTING) | defined(USE_STATIC_REFLECTION) || defined(USE_DYNAMIC_REFLECTION)

#ifdef USE_SPECULAR_LIGHTING
	mediump vec3 halfVector = normalize(Light0DirectionOS + vV);
	specUV = vec2(max(0.0, dot(netNormal.xyz, halfVector)), 0.0);
#endif

#if defined(USE_STATIC_REFLECTION) || defined(USE_DYNAMIC_REFLECTION)
	// calculate lighting
	lowp vec4 vR = vec4(reflect(-vV, netNormal.xyz), 0.0);
	vCoordFalloff = (WorldViewIT * vec4(netNormal, 0.0)).xy * vec2(0.49, -0.49) + vec2(0.5, 0.5);
#endif
	
#ifdef USE_STATIC_REFLECTION
	vCoordCubeMap = (World * vR).xyz;
#endif

#ifdef USE_DYNAMIC_REFLECTION
	//Compute the reflection vector
	vCoordFrameBuffer = vec2(0.46875, 0.3125) + vec2(0.46875, -0.3125) * (WorldViewIT * vec4(vR)).xy;
#endif 

#endif
	
#endif // SKINNED


/////////////////////////
//Fog factor output	   //
/////////////////////////
#ifdef FG
#if defined(SKINNED) && !defined(DCC_MAX)
	highp float fogDistance = dot(matWorldViewT[2], netPosition);
#else
	highp float fogDistance = dot(matWorldViewT[2], Vertex);
#endif
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
	
	vCoord0 = TexCoord0;

}