//#define VERTEX_SHADER
//#define FRAGMENT_SHADER
//#define DEBUG_RENDER_CASCADE
//#define RENDER_CASCADE
//#define DEBUG_LIGHT_AND_SHADOW
//#define LIGHT_AND_SHADOW
//#define DEBUG_LIGHT_AND_SHADOW_HW
//#define LIGHT_AND_SHADOW_HW

// ------------------------------------------------------------
// ----                DEBUG_RENDER_CASCADE                ----
// ----  Fill shadow map with debug color for each cascade ----
// ------------------------------------------------------------

#ifdef DEBUG_RENDER_CASCADE

uniform highp mat4 WorldViewProjectionMatrix;

#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

void main()
{
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform lowp int cascadeIndex;

#ifdef FRAGMENT_SHADER
void main()
{
	const lowp vec4 red = vec4(1.0, 0.0, 0.0, 1.0);
	const lowp vec4 blue = vec4(0.0, 0.0, 1.0, 1.0);
	const lowp vec4 green = vec4(0.0, 1.0, 0.0, 1.0);
	const lowp vec4 purple = vec4(1.0, 0.0, 1.0, 1.0);
	const lowp vec4 yellow = vec4(1.0, 1.0, 0.0, 1.0);
	
	// Ugly unoptimal if..else.. but don't care since it is just for debug
	lowp vec4 debugColor;
	
	if (cascadeIndex == 0)
		debugColor = red;
	else if (cascadeIndex == 1)
		debugColor = green;
	else if (cascadeIndex == 2)
		debugColor = blue;
	else if (cascadeIndex == 3)
		debugColor = purple;
	else // Shouldn't happen
		debugColor = yellow;
	
	gl_FragColor = debugColor;
}
#endif
// FRAGMENT_SHADER

#endif 
// DEBUG_RENDER_CASCADE


// ------------------------------------------------------------
// ----                   RENDER_CASCADE                   ----
// ----  Fill shadow map with no specific color            ----
// ------------------------------------------------------------
#ifdef RENDER_CASCADE

uniform highp mat4 WorldViewProjectionMatrix;

#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

void main()
{
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

#ifdef FRAGMENT_SHADER
void main()
{
	gl_FragColor = vec4(1.0,0.0,0.0,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// RENDER_CASCADE

// ------------------------------------------------------------
// ----              DEBUG_LIGHT_AND_SHADOW                ----
// ----  Render objects with basic diffuse lighting        ----
// ----  and color coded cascade index                     ----
// ------------------------------------------------------------

#ifdef DEBUG_LIGHT_AND_SHADOW

uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldView;
uniform highp mat4 WorldIT;
uniform highp mat4 World;


#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		vec3	vPosVS;		// View Space position
varying highp		vec4	vPosWS;		// World Space position

void main()
{
	vPosWS = World * Vertex;
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	vPosVS = (WorldView * Vertex).xyz;
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec2 camnearandfar;
uniform mediump vec3 cascadeFar;

uniform mediump vec3 sunLightDir; // WS normalized

uniform highp mat4 ViewToLightMatrix0;
uniform highp mat4 LightProjMatrix0;

uniform highp mat4 ViewToLightMatrix1;
uniform highp mat4 LightProjMatrix1;

uniform highp mat4 ViewToLightMatrix2;
uniform highp mat4 LightProjMatrix2;

uniform highp mat4 ViewToLightMatrix3;
uniform highp mat4 LightProjMatrix3;

uniform highp sampler2D CascadedShadowMap;

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space normal
varying highp		vec3	vPosVS;		// View Space position
varying highp		vec4    vPosWS;		// World Space position

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// TODO: optimize ?
lowp int computeCascadeIndex() 
{
	int index = 0;
	
	lowp vec3 comp = step(cascadeFar,vec3(abs(vPosVS.z)));
	index = int(dot(vec3(1.0),comp));
	
	return index;
}

// Return 1.0 if visible, 0.0 if occluded
mediump float computeVisibility(int index)
{
	highp float bias = 0.001 * float(index + 1);
	const highp float atlasScale = 0.5;
	
	highp vec2 atlasShift = vec2(0.0,0.0);
	highp mat4 LightProjMatrix = LightProjMatrix0;
	highp mat4 ViewToLightMatrix = ViewToLightMatrix0;
	
	if (index == 1) {
		atlasShift = vec2(0.0,0.5);
		LightProjMatrix = LightProjMatrix1;
		ViewToLightMatrix = ViewToLightMatrix1;
	}
	else if (index == 2) {
		atlasShift = vec2(0.5,0.0);
		LightProjMatrix = LightProjMatrix2;
		ViewToLightMatrix = ViewToLightMatrix2;
	}
	else if (index == 3) {
		atlasShift = vec2(0.5,0.5);
		LightProjMatrix = LightProjMatrix3;
		ViewToLightMatrix = ViewToLightMatrix3;
	}
	
	// Compute vertex position in LS
	highp vec4 posLS = LightProjMatrix * (ViewToLightMatrix * vPosWS);
	
	posLS.xyz = posLS.xyz * 0.5 + 0.5;
	posLS.y = 1.0 - posLS.y;
	posLS.xy = posLS.xy * atlasScale + atlasShift;
	
	posLS.z -= bias;
	
	// Fetch proper cascade
	highp float currentDepth = texture2D(CascadedShadowMap, posLS.xy).r;
	
	return float(currentDepth > posLS.z);
	
}

void main()
{
	const lowp vec3 red = vec3(1.0, 0.3, 0.3);
	const lowp vec3 blue = vec3(0.3, 0.3, 1.0);
	const lowp vec3 green = vec3(0.3, 1.0, 0.3);
	const lowp vec3 purple = vec3(1.0, 0.3, 1.0);
	const lowp vec3 yellow = vec3(1.0, 1.0, 0.3);
	
	mediump float lightIntensity = 0.0;
	lowp int cascadeIndex = 0;
	
	lightIntensity = computeLighting();
	cascadeIndex = computeCascadeIndex();
	
	highp float visibility = computeVisibility(cascadeIndex);
	
	// Ugly unoptimal if..else.. but don't care since it is just for debug
	lowp vec3 cascadeColor;
	
	if (cascadeIndex == 0)
		cascadeColor = red;
	else if (cascadeIndex == 1)
		cascadeColor = green;
	else if (cascadeIndex == 2)
		cascadeColor = blue;
	else if (cascadeIndex == 3)
		cascadeColor = purple;
	else // Shouldn't happen
		cascadeColor = yellow;
		
	const lowp vec4 shadowColor = vec4(0.0,0.0,0.0,0.8);
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	//shadowIntensity = min(0.0,shadowIntensity);
	lowp vec3 finalColor = mix(cascadeColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// DEBUG_LIGHT_AND_SHADOW

// ------------------------------------------------------------
// ----                 LIGHT_AND_SHADOW                   ----
// ----  Render objects with basic diffuse lighting        ----
// ----  and shadows from cascade                          ----
// ------------------------------------------------------------

#ifdef LIGHT_AND_SHADOW

uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldView;
uniform highp mat4 WorldIT;

#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		float	vDepthVS;	// View Space

void main()
{
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	vDepthVS = abs((WorldView * Vertex).z);
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec2 camnearandfar;
uniform mediump vec3 cascadeFar;

uniform mediump vec3 sunLightDir; // already normalized

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space
varying highp		float	vDepthVS;	// View Space

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// TODO: optimize ?
lowp int computeCascadeIndex() 
{	
	int index = 0;
	
	lowp vec3 comp = step(cascadeFar,vec3(vDepthVS));
	index = int(dot(vec3(1.0),comp));
	
	return index;
}

// Return 1.0 if visible, 0.0 if occluded
mediump float computeVisibility(int index)
{
	// TODO
	return 1.0 - 0.0001 * float(index);
}

void main()
{
	mediump float lightIntensity = 0.0;
	mediump float visibility = 0.0;	// 1.0 for visible, 0.0 for occluded
	lowp int cascadeIndex = 0;
	
	lightIntensity = computeLighting();
	cascadeIndex = computeCascadeIndex();
	visibility = computeVisibility(cascadeIndex);
	
	// B&W result for now
	// TODO: get material diffuse color / texture
	// ?? Or only output visibility result ??
	const lowp vec3 diffuseColor = vec3(1.0);
	
	const lowp vec4 shadowColor = vec4(0.0,0.0,0.0,0.8);
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	lowp vec3 finalColor = mix(diffuseColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// LIGHT_AND_SHADOW

// ------------------------------------------------------------
// ----          DEBUG_LIGHT_AND_SHADOW_HW                 ----
// ----  Render objects with basic diffuse lighting        ----
// ----  and color coded cascade index                     ----
// ------------------------------------------------------------

#ifdef DEBUG_LIGHT_AND_SHADOW_HW

#if GL_ES
#extension GL_EXT_shadow_samplers : require
#endif


uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldView;
uniform highp mat4 WorldIT;
uniform highp mat4 World;


#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		vec3	vPosVS;		// View Space position
varying highp		vec4	vPosWS;		// World Space position

void main()
{
	vPosWS = World * Vertex;
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	vPosVS = (WorldView * Vertex).xyz;
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec2 camnearandfar;
uniform mediump vec3 cascadeFar;

uniform mediump vec3 sunLightDir; // WS normalized

uniform highp mat4 ViewToLightMatrix0;
uniform highp mat4 LightProjMatrix0;

uniform highp mat4 ViewToLightMatrix1;
uniform highp mat4 LightProjMatrix1;

uniform highp mat4 ViewToLightMatrix2;
uniform highp mat4 LightProjMatrix2;

uniform highp mat4 ViewToLightMatrix3;
uniform highp mat4 LightProjMatrix3;

//uniform highp sampler2D CascadedShadowMap; 
uniform highp sampler2DShadow CascadedShadowMap;

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space normal
varying highp		vec3	vPosVS;		// View Space position
varying highp		vec4    vPosWS;		// World Space position

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// TODO: optimize ?
lowp int computeCascadeIndex() 
{
	int index = 0;
	
	lowp vec3 comp = step(cascadeFar,vec3(abs(vPosVS.z)));
	index = int(dot(vec3(1.0),comp));
	
	return index;
}

// Return 1.0 if visible, 0.0 if occluded
mediump float computeVisibility(int index)
{
	highp float bias = 0.001 * float(index + 1);
	const highp float atlasScale = 0.5;
	
	highp vec2 atlasShift = vec2(0.0,0.0);
	highp mat4 LightProjMatrix = LightProjMatrix0;
	highp mat4 ViewToLightMatrix = ViewToLightMatrix0;
	
	if (index == 1) {
		atlasShift = vec2(0.0,0.5);
		LightProjMatrix = LightProjMatrix1;
		ViewToLightMatrix = ViewToLightMatrix1;
	}
	else if (index == 2) {
		atlasShift = vec2(0.5,0.0);
		LightProjMatrix = LightProjMatrix2;
		ViewToLightMatrix = ViewToLightMatrix2;
	}
	else if (index == 3) {
		atlasShift = vec2(0.5,0.5);
		LightProjMatrix = LightProjMatrix3;
		ViewToLightMatrix = ViewToLightMatrix3;
	}
	
	// Compute vertex position in LS
	highp vec4 posLS = LightProjMatrix * (ViewToLightMatrix * vPosWS);
	
	posLS.xyz = posLS.xyz * 0.5 + 0.5;
	posLS.y = 1.0 - posLS.y;
	posLS.xy = posLS.xy * atlasScale + atlasShift;
	
	posLS.z -= bias;
	
	// Fetch proper cascade
	//highp float currentDepth = texture2D(CascadedShadowMap, posLS.xy).r;
	//return float(currentDepth > posLS.z - bias);
	
	// Use hardware SM
#if GL_ES
	return shadow2DEXT(CascadedShadowMap, posLS.xyz);
#else
	return shadow2D(CascadedShadowMap, posLS.xyz).r;
#endif
	
}

void main()
{
	const lowp vec3 red = vec3(1.0, 0.3, 0.3);
	const lowp vec3 blue = vec3(0.3, 0.3, 1.0);
	const lowp vec3 green = vec3(0.3, 1.0, 0.3);
	const lowp vec3 purple = vec3(1.0, 0.3, 1.0);
	const lowp vec3 yellow = vec3(1.0, 1.0, 0.3);
	
	mediump float lightIntensity = 0.0;
	lowp int cascadeIndex = 0;
	
	lightIntensity = computeLighting();
	cascadeIndex = computeCascadeIndex();
	
	highp float visibility = computeVisibility(cascadeIndex);
	
	// Ugly unoptimal if..else.. but don't care since it is just for debug
	lowp vec3 cascadeColor;
	
	if (cascadeIndex == 0)
		cascadeColor = red;
	else if (cascadeIndex == 1)
		cascadeColor = green;
	else if (cascadeIndex == 2)
		cascadeColor = blue;
	else if (cascadeIndex == 3)
		cascadeColor = purple;
	else // Shouldn't happen
		cascadeColor = yellow;
	
		
	const lowp vec4 shadowColor = vec4(0.0,0.0,0.0,0.8);
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	//shadowIntensity = min(0.0,shadowIntensity);
	lowp vec3 finalColor = mix(cascadeColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif
// DEBUG_LIGHT_AND_SHADOW_HW


// ------------------------------------------------------------
// ----              LIGHT_AND_SHADOW_HW                   ----
// ----  Render objects with basic diffuse lighting        ----
// ----  and color coded cascade index                     ----
// ------------------------------------------------------------

#ifdef LIGHT_AND_SHADOW_HW

#if GL_ES
#extension GL_EXT_shadow_samplers : require
#endif


uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldView;
uniform highp mat4 WorldIT;
uniform highp mat4 World;


#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		vec3	vPosVS;		// View Space position
varying highp		vec4	vPosWS;		// World Space position

void main()
{
	vPosWS = World * Vertex;
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	vPosVS = (WorldView * Vertex).xyz;
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec2 camnearandfar;
uniform mediump vec3 cascadeFar;

uniform mediump vec3 sunLightDir; // WS normalized

uniform highp mat4 ViewToLightMatrix0;
uniform highp mat4 LightProjMatrix0;

uniform highp mat4 ViewToLightMatrix1;
uniform highp mat4 LightProjMatrix1;

uniform highp mat4 ViewToLightMatrix2;
uniform highp mat4 LightProjMatrix2;

uniform highp mat4 ViewToLightMatrix3;
uniform highp mat4 LightProjMatrix3;

//uniform highp sampler2D CascadedShadowMap; 
uniform highp sampler2DShadow CascadedShadowMap;

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space normal
varying highp		vec3	vPosVS;		// View Space position
varying highp		vec4    vPosWS;		// World Space position

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// TODO: optimize ?
lowp int computeCascadeIndex() 
{
	int index = 0;
	
	lowp vec3 comp = step(cascadeFar,vec3(abs(vPosVS.z)));
	index = int(dot(vec3(1.0),comp));
	
	return index;
}

// Return 1.0 if visible, 0.0 if occluded
// TODO: optim: precompute matrix product
mediump float computeVisibility(int index)
{
	highp float bias = 0.001 * float(index + 1);
	const highp float atlasScale = 0.5;
	
	highp vec2 atlasShift = vec2(0.0,0.0);
	highp mat4 LightProjMatrix = LightProjMatrix0;
	highp mat4 ViewToLightMatrix = ViewToLightMatrix0;
	
	if (index == 1) {
		atlasShift = vec2(0.0,0.5);
		LightProjMatrix = LightProjMatrix1;
		ViewToLightMatrix = ViewToLightMatrix1;
	}
	else if (index == 2) {
		atlasShift = vec2(0.5,0.0);
		LightProjMatrix = LightProjMatrix2;
		ViewToLightMatrix = ViewToLightMatrix2;
	}
	else if (index == 3) {
		atlasShift = vec2(0.5,0.5);
		LightProjMatrix = LightProjMatrix3;
		ViewToLightMatrix = ViewToLightMatrix3;
	}
	
	// Compute vertex position in LS
	highp vec4 posLS = LightProjMatrix * (ViewToLightMatrix * vPosWS);
	
	posLS.xyz = posLS.xyz * 0.5 + 0.5;
	posLS.y = 1.0 - posLS.y;
	posLS.xy = posLS.xy * atlasScale + atlasShift;
	
	posLS.z -= bias;
	
	// Fetch proper cascade
	//highp float currentDepth = texture2D(CascadedShadowMap, posLS.xy).r;
	//return float(currentDepth > posLS.z - bias);
	
	// Use hardware SM
#if GL_ES
	return shadow2DEXT(CascadedShadowMap, posLS.xyz);
#else
	return shadow2D(CascadedShadowMap, posLS.xyz).r;
#endif
	
}

void main()
{
	mediump float lightIntensity = 0.0;
	lowp int cascadeIndex = 0;
	
	lightIntensity = computeLighting();
	cascadeIndex = computeCascadeIndex();
	
	highp float visibility = computeVisibility(cascadeIndex);
	
	const lowp vec3 diffuseColor = (0.8,0.8,0.8);
	
	const lowp vec4 shadowColor = vec4(0.0,0.0,0.0,0.8);
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	//shadowIntensity = min(0.0,shadowIntensity);
	lowp vec3 finalColor = mix(diffuseColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif
// LIGHT_AND_SHADOW_HW
