#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

#define USE_CUSTOM_LIGHT_0
#define USE_CUSTOM_LIGHT_1
#define USE_CUSTOM_LIGHT_2

#ifdef AT
	uniform lowp float alpharef;
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

varying lowp vec4 vColor DCC(: COLOR0);

#ifdef USE_CUSTOM_LIGHT_0
	varying mediump vec4 CustomLight0HalfVector;
	uniform lowp vec4 CustomLight0SpecularColor;
	uniform mediump float CustomLight0SpecularFactor;
	uniform mediump float CustomLight0SpecularGlossiness;
#endif

#ifdef USE_CUSTOM_LIGHT_1
	varying mediump vec4 CustomLight1HalfVector;
	uniform lowp vec4 CustomLight1SpecularColor;
	uniform mediump float CustomLight1SpecularFactor;
	uniform mediump float CustomLight1SpecularGlossiness;
#endif

#ifdef USE_CUSTOM_LIGHT_2
	varying mediump vec4 CustomLight2HalfVector;
	uniform lowp vec4 CustomLight2SpecularColor;
	uniform mediump float CustomLight2SpecularFactor;
	uniform mediump float CustomLight2SpecularGlossiness;
#endif

uniform lowp sampler2D DiffuseSampler;
uniform lowp sampler2D SpecularSampler;

#ifdef AL
	uniform lowp float Object_Alpha;
#endif
	      
uniform lowp sampler2D NormalSampler;

#ifdef FG
	varying lowp float vFogFactor;
	uniform lowp vec4 fogcolor;   
#endif

uniform lowp sampler2D ColorVarMask;
uniform lowp vec3 ColorVar1;
uniform lowp vec3 ColorVar2;
uniform lowp vec3 ColorVar3;
uniform lowp vec3 ColorVar4;

#define TONED
#ifdef TONED
	uniform lowp vec3 ToneGain;
	uniform lowp vec3 ToneOffset;
#endif

void main()
{
	//Dependant fetch FTL... but the main character is at mipmap 2/3 from default camera distance...
	lowp vec4 Diffuse = texture2D(DiffuseSampler, vTexCoord0, -3.0);
	
	lowp vec4 ColorMask = texture2D(ColorVarMask, vTexCoord0);
	
	lowp vec3 diffuseColor = Diffuse.rgb;
	diffuseColor += ColorVar1 * ColorMask.x;
	diffuseColor += ColorVar2 * ColorMask.y;
	diffuseColor += ColorVar3 * ColorMask.z;
	diffuseColor += ColorVar4 * ColorMask.w;
	
	Diffuse.rgb = clamp(diffuseColor, 0.0, 1.0);
	
	lowp vec3 nNormal =  (texture2D(NormalSampler, vTexCoord0) * 2.0 - 1.0).xyz;
	nNormal = normalize(nNormal);
	
	lowp vec4 lightColor = vColor;

	lowp vec4 color = lightColor * Diffuse;

	mediump vec3 TotalSpecularColor = vec3(0.0, 0.0, 0.0);

#ifdef USE_CUSTOM_LIGHT_0
	//LambertPower is now combined with Glossiness code side, to remove one pow operation

	mediump vec3 FinalCustomLight0SpecularColor = CustomLight0SpecularColor.rgb;
	mediump vec3 FinalCustomLight0HalfVector = normalize(CustomLight0HalfVector.xyz);
	mediump float CustomLight0HalfLamb = (dot(nNormal, FinalCustomLight0HalfVector) * 0.5) + 0.5;
	mediump float CustomLight0AttNdotHV =  pow(CustomLight0HalfLamb, CustomLight0SpecularGlossiness);

	FinalCustomLight0SpecularColor *= CustomLight0SpecularFactor * CustomLight0AttNdotHV * CustomLight0HalfVector.w;
	
	TotalSpecularColor += FinalCustomLight0SpecularColor;
#endif

#ifdef USE_CUSTOM_LIGHT_1
	//LambertPower is now combined with Glossiness code side, to remove one pow operation

	mediump vec3 FinalCustomLight1SpecularColor = CustomLight1SpecularColor.rgb;
	mediump vec3 FinalCustomLight1HalfVector = normalize(CustomLight1HalfVector.xyz);
	mediump float CustomLight1HalfLamb = (dot(nNormal, FinalCustomLight1HalfVector) * 0.5) + 0.5;
	mediump float CustomLight1AttNdotHV =  pow(CustomLight1HalfLamb, CustomLight1SpecularGlossiness);

	FinalCustomLight1SpecularColor *= CustomLight1SpecularFactor * CustomLight1AttNdotHV * CustomLight1HalfVector.w;
	
	TotalSpecularColor += FinalCustomLight1SpecularColor;
#endif

#ifdef USE_CUSTOM_LIGHT_2
	//LambertPower is now combined with Glossiness code side, to remove one pow operation

	mediump vec3 FinalCustomLight2SpecularColor = CustomLight2SpecularColor.rgb;
	mediump vec3 FinalCustomLight2HalfVector = normalize(CustomLight2HalfVector.xyz);
	mediump float CustomLight2HalfLamb = (dot(nNormal, FinalCustomLight2HalfVector) * 0.5) + 0.5;
	mediump float CustomLight2AttNdotHV =  pow(CustomLight2HalfLamb, CustomLight2SpecularGlossiness);

	FinalCustomLight2SpecularColor *= CustomLight2SpecularFactor * CustomLight2AttNdotHV * CustomLight2HalfVector.w;
	
	TotalSpecularColor += FinalCustomLight2SpecularColor;
#endif
 
 	lowp vec3 SpecMap = texture2D(SpecularSampler, vTexCoord0).rgb;
	TotalSpecularColor = TotalSpecularColor * SpecMap.r;

	color.rgb += TotalSpecularColor;
	
#ifdef AL
	color.a *= Object_Alpha;
#endif

#ifdef FG
	color.xyz = mix(color.xyz, fogcolor.xyz, vFogFactor);
#endif

#ifdef TONED	
	color.rgb = color.rgb * ToneGain + ToneOffset;
#endif

	gl_FragColor = clamp(color, 0.0, 1.0);
} // main end
