#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define SKINNED
//#define FG

#define USE_CUSTOM_LIGHT_0
#define USE_CUSTOM_LIGHT_1
#define USE_CUSTOM_LIGHT_2

attribute highp vec4 Vertex DCC(: POSITION);
attribute highp vec3 Normal DCC(: NORMAL);
attribute highp vec3 Binormal DCC(: BINORMAL);
attribute highp vec3 Tangent DCC(: TANGENT);
attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);
varying lowp vec4 vColor DCC(: COLOR0);

uniform highp vec3 eyeposition;

uniform highp mat4 matWorldViewProjection;
uniform highp mat4 matWorld;
//uniform highp mat4 matWorldIT;
//uniform highp mat4 matWorldT;
//uniform highp mat4 matViewIT;

uniform lowp vec4 Light0Ambientcolor;

uniform mediump vec4 irdcAr; 
uniform mediump vec4 irdcAg;
uniform mediump vec4 irdcAb;
uniform mediump vec4 irdcBr;
uniform mediump vec4 irdcBg;
uniform mediump vec4 irdcBb;
uniform mediump vec3 irdcC;
uniform highp vec3 irdSpecDir;

uniform mediump vec3 IrrAmbientDiffuseMult;

#if defined(SKINNED) && !defined(DCC_MAX)
	uniform highp mat4 BoneMatrices[48];
	uniform highp vec4 WeightMask;
	attribute mediump vec4 SkinWeights;
	attribute mediump vec4 SkinIndices;
#endif

#ifdef USE_CUSTOM_LIGHT_0
	uniform mediump vec3 CustomLight0Position;
	uniform mediump vec3 CustomLight0Direction;
	uniform mediump float CustomLight0Attenuation;
	varying mediump vec4 CustomLight0HalfVector;
#endif

#ifdef USE_CUSTOM_LIGHT_1
	uniform mediump vec3 CustomLight1Position;
	uniform mediump vec3 CustomLight1Direction;
	uniform mediump float CustomLight1Attenuation;
	varying mediump vec4 CustomLight1HalfVector;
#endif

#ifdef USE_CUSTOM_LIGHT_2
	uniform mediump vec3 CustomLight2Position;
	uniform mediump vec3 CustomLight2Direction;
	uniform mediump float CustomLight2Attenuation;
	varying mediump vec4 CustomLight2HalfVector;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

//This shader will not work properly on an unskinned mesh, as we assume that the attributes are already in World Space.
void main()
{
#if defined(SKINNED) && !defined(DCC_MAX)
    highp vec4 weights = SkinWeights * WeightMask;
    highp vec4 netPosition = vec4(0.0, 0.0, 0.0, 0.0);
    highp vec4 netNormal = vec4(0.0, 0.0, 0.0, 0.0);
	highp vec4 netTangent = vec4(0.0, 0.0, 0.0, 0.0);
	highp vec4 netBinormal = vec4(0.0, 0.0, 0.0, 0.0);

	vec4 indices = SkinIndices;

    for (int i = 0; i < 4; i++)
    {
        int index = int(indices[i]);
        netPosition += weights[i] * (BoneMatrices[index] * vec4(Vertex.xyz, 1.0));
        netNormal += weights[i] * (BoneMatrices[index] * vec4(Normal.xyz, 0.0));
        netTangent += weights[i] * (BoneMatrices[index] * vec4(Tangent.xyz, 0.0)); 
        netBinormal += weights[i] * (BoneMatrices[index] * vec4(Binormal.xyz, 0.0));     
    }
    
   	// netNormal.xyz = normalize(netNormal.xyz);
    
    // Vertex Position output
	gl_Position = matWorldViewProjection * netPosition;
#else
	// Vertex Position output
	gl_Position = matWorldViewProjection * Vertex;

	highp vec4 netPosition = matWorld * Vertex;	

	#if !defined(DCC_MAX)
		highp vec4 netNormal = matWorld * vec4(Normal, 0.0);
		highp vec4 netTangent = matWorld * vec4(Tangent, 0.0);		
		highp vec4 netBinormal = matWorld * vec4(Binormal, 0.0);

		// netNormal.xyz = normalize(netNormal.xyz);
		// netTangent.xyz = normalize(netTangent.xyz);
		// netBinormal.xyz = normalize(netBinormal.xyz);			
	#endif
#endif

	mediump vec3 posToCam = eyeposition - netPosition.xyz;
	mediump vec3 eye = normalize(posToCam);

#if defined(DCC_MAX)
	mediump mat3 objTangentXf = mat3(
							Binormal.x, -Tangent.x, Normal.x,
        	                Binormal.y, -Tangent.y, Normal.y,
            	            Binormal.z, -Tangent.z, Normal.z
						);
						
	IrrAmbientDiffuseMult = min(vec3(0.0, 0.0, 0.0), IrrAmbientDiffuseMult);
    IrrAmbientDiffuseMult = max(vec3(0.0, 0.0, 0.0), IrrAmbientDiffuseMult);
	vColor = vec4(1.0);
	vColor.xyz += IrrAmbientDiffuseMult;
	
	if (vColor.rgb == vec3(0.0))
    	vColor = vec4(1.0);
#else
	mediump mat3 objTangentXf = mat3(
						netTangent.x, netBinormal.x, netNormal.x,
    	                netTangent.y, netBinormal.y, netNormal.y,
        	            netTangent.z, netBinormal.z, netNormal.z
					);

	mediump vec4 normal = vec4(netNormal.xyz, 1.0);
		
	normal.yz = normal.zy;
	normal.y *= -1.0;	
		
	mediump vec3 x1, x2, x3;
    
	// Linear + constant polynomial terms
    x1.r = dot(irdcAr,normal);
	x1.g = dot(irdcAg,normal);
	x1.b = dot(irdcAb,normal);
	    
	// 4 of the quadratic polynomials
	mediump vec4 vB = normal.xyzz * normal.yzzx;   
	x2.r = dot(irdcBr,vB);
	x2.g = dot(irdcBg,vB);
	x2.b = dot(irdcBb,vB);
	   
	// Final quadratic polynomial
	mediump float vC = normal.x*normal.x - normal.y*normal.y;
	x3 = irdcC.rgb * vC;    
	
	lowp vec3 diffuse = x1 + x2 + x3;
	    
	diffuse = max(vec3(0.0, 0.0, 0.0), diffuse);
	
	lowp vec3 finalAmbientDiffuseColor =  Light0Ambientcolor.xyz * IrrAmbientDiffuseMult.x + diffuse * IrrAmbientDiffuseMult.y;
	
	vColor = vec4(finalAmbientDiffuseColor, 1.0);
#endif

#ifdef USE_CUSTOM_LIGHT_0
	mediump vec3 posToCustomLight0 = CustomLight0Position - netPosition.xyz;
	mediump float distanceToCustomLight0 = length(posToCustomLight0);
	mediump float attenuationCustomLight0 = smoothstep(0.0,  CustomLight0Attenuation, distanceToCustomLight0);
	CustomLight0HalfVector.xyz = objTangentXf * normalize(CustomLight0Direction + eye);
	CustomLight0HalfVector.w = 1.0 - attenuationCustomLight0;
#endif

#ifdef USE_CUSTOM_LIGHT_1
	mediump vec3 posToCustomLight1 = CustomLight1Position - netPosition.xyz;
	mediump float distanceToCustomLight1 = length(posToCustomLight1);
	mediump float attenuationCustomLight1 = smoothstep(0.0, CustomLight1Attenuation, distanceToCustomLight1);
	CustomLight1HalfVector.xyz = objTangentXf * normalize(CustomLight1Direction + eye);
	CustomLight1HalfVector.w = 1.0 - attenuationCustomLight1;
#endif

#ifdef USE_CUSTOM_LIGHT_2
	mediump vec3 posToCustomLight2 = CustomLight2Position - netPosition.xyz;
	mediump float distanceToCustomLight2 = length(posToCustomLight2);
	mediump float attenuationCustomLight2 = smoothstep(0.0, CustomLight2Attenuation, distanceToCustomLight2);
	CustomLight2HalfVector.xyz = objTangentXf * normalize(CustomLight2Direction + eye);
	CustomLight2HalfVector.w = 1.0 - attenuationCustomLight2;
#endif

	vTexCoord0 = texcoord0;	// Diffuse/Specular
	
#ifdef FG
	float fogDistance = length(posToCam);
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
} // main end
