uniform lowp mat4 ColorMatrix; // Android

#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

#if defined(REQUEST_USE_BACK_LIGHT) && !defined(BYPASS_USE_BACK_LIGHT)
#define USE_BACK_LIGHT
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

varying lowp vec4 vColor DCC(: COLOR0);

uniform lowp vec4 Light0DiffuseColor;
uniform lowp vec4 Light1DiffuseColor;

#ifdef USE_BACK_LIGHT
varying mediump vec3 BacklightHalfVector;
uniform lowp vec4 BacklightDiffuseColor;
uniform lowp vec4 BacklightSpecularColor;
uniform mediump float BacklightSpecularFactor;
uniform mediump float BacklightSpecularGlossiness;
#endif // USE_BACK_LIGHT

varying mediump vec3 vVP[2];

uniform lowp vec4 Light0Specularcolor;

uniform lowp sampler2D DiffuseSampler;
uniform lowp sampler2D SpecularSampler;

#ifdef AL
	uniform lowp float Object_Alpha;
#endif

uniform mediump float Glossiness;

varying mediump vec3 vHalfVector;
uniform lowp float SpecFactor;
uniform lowp vec4 SpecColor;

uniform lowp sampler2D NormalSampler;

uniform lowp float SunOcclusionFactor;

uniform lowp vec4 sunFactor;

#ifdef FG
	varying lowp float vFogFactor;
	uniform lowp vec4 fogcolor;   
#endif

uniform lowp sampler2D ColorVarMask;
uniform lowp sampler2D ColorVarMask_Alpha;
uniform lowp vec3 ColorVar1;
uniform lowp vec3 ColorVar2;
uniform lowp vec3 ColorVar3;
uniform lowp vec3 ColorVar4;

#define TONED
#ifdef TONED
	uniform lowp vec3 ToneGain;
	uniform lowp vec3 ToneOffset;
#endif

void main()
{
	//Dependant fetch FTL... but the main character is at mipmap 2/3 from default camera distance...
		lowp vec4 Diffuse = texture2D(DiffuseSampler, vTexCoord0); //Android Luis avoid i9000 eternal draw
	
	lowp vec4 ColorMask = texture2D(ColorVarMask, vTexCoord0);
	ColorMask.a = texture2D(ColorVarMask_Alpha, vTexCoord0).r; //Android
	
	lowp vec3 diffuseColor = Diffuse.rgb;
	diffuseColor += ColorVar1 * ColorMask.x;
	diffuseColor += ColorVar2 * ColorMask.y;
	diffuseColor += ColorVar3 * ColorMask.z;
	diffuseColor += ColorVar4 * ColorMask.w;
	
	Diffuse.rgb = clamp(diffuseColor, 0.0, 1.0);
	
	lowp vec3 nNormal =  (texture2D(NormalSampler, vTexCoord0) * 2.0 - 1.0).xyz;
	nNormal = normalize(nNormal);
	lowp vec4 DiffuseLight = vec4(0.0, 0.0, 0.0, 0.0);
		
	lowp float nDotVP0 = max(0.0, dot(nNormal, vVP[0].xyz));
	DiffuseLight += Light0DiffuseColor * nDotVP0;
	lowp float nDotVP1 = max(0.0, dot(nNormal, vVP[1].xyz));
	DiffuseLight += Light1DiffuseColor * nDotVP1;

#ifdef DCC_MAX
	SunOcclusionFactor = max(SunOcclusionFactor, 1.0);
#endif
	
#ifdef USE_BACK_LIGHT
	lowp float nDotVP2 = max(0.0, dot(nNormal, -vVP[0].xyz));
	DiffuseLight += BacklightDiffuseColor * nDotVP2;
#endif // USE_BACK_LIGHT	

	// Specular lighting will be calculated in the fragment
    lowp vec4 lightColor = vec4(vColor.rgb + DiffuseLight.rgb * SunOcclusionFactor, vColor.a);

	lowp vec4 color = lightColor * Diffuse;

	lowp vec3 SpecLight = Light0Specularcolor.xyz;

	mediump vec3 halfVector = normalize(vHalfVector);
	mediump float HalfLamb = (dot(nNormal, halfVector) * 0.5) + 0.5;
    //HalfLamb = pow(HalfLamb, LambertPower);
    //halfLamb will be between 0 and 1, no need to "max" it...
    //LambertPower is now combined with Glossiness code side, to remove one pow operation
    lowp float attNdotHV =  pow(HalfLamb, Glossiness);
    
   	SpecLight = SpecColor.xyz;
	
	lowp vec3 SpecMap = texture2D(SpecularSampler, vTexCoord0).rgb;

	lowp vec3 specularColor = SpecLight * SpecFactor * attNdotHV;
		
#ifdef USE_BACK_LIGHT
	mediump vec3 normalizedBacklightHalfVector = normalize(BacklightHalfVector);
	mediump float halfBacklightLamb = (dot(nNormal, normalizedBacklightHalfVector) * 0.5) + 0.5;
	mediump float attBacklightNdotHV =  pow(halfBacklightLamb, BacklightSpecularGlossiness);
	lowp vec3 finalBacklightSpecularColor = BacklightSpecularColor.rgb * BacklightSpecularFactor * attBacklightNdotHV;
	
#ifdef DCC_MAX
	finalBacklightSpecularColor.r = max(finalBacklightSpecularColor.r, 1.0);
	finalBacklightSpecularColor.g = max(finalBacklightSpecularColor.g, 1.0);
	finalBacklightSpecularColor.b = max(finalBacklightSpecularColor.b, 1.0);	
#endif
	
#endif // USE_BACK_LIGHT	

	color.rgb += (
				specularColor * SunOcclusionFactor
#ifdef USE_BACK_LIGHT
				+ finalBacklightSpecularColor
#endif // USE_BACK_LIGHT
			) * SpecMap.r;

#ifdef AL
	color.a *= Object_Alpha;
#endif

	color.rgb += sunFactor.rgb;

#ifdef FG
	color.xyz = mix(color.xyz, fogcolor.xyz, vFogFactor);
#endif

#ifdef TONED	
	color.rgb = color.rgb * ToneGain + ToneOffset;
#endif

	gl_FragColor = clamp(color, 0.0, 1.0);
} // main end
