#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define SKINNED
//#define FG

#if defined(REQUEST_USE_BACK_LIGHT) && !defined(BYPASS_USE_BACK_LIGHT)
#define USE_BACK_LIGHT
#endif

attribute highp vec4 Vertex DCC(: POSITION);
attribute highp vec3 Normal DCC(: NORMAL);
attribute highp vec3 Binormal DCC(: BINORMAL);
attribute highp vec3 Tangent DCC(: TANGENT);
attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

varying lowp vec4 vColor DCC(: COLOR0);

varying mediump vec3 vHalfVector;

#ifdef USE_BACK_LIGHT
varying mediump vec3 BacklightHalfVector;
#endif // USE_BACK_LIGHT

varying mediump vec3 vVP[2];

uniform highp vec3 eyeposition;

uniform lowp float SunOcclusionFactor;

uniform highp mat4 matWorldViewProjection;
uniform highp mat4 matWorld;					// Lighting
uniform highp mat4 matWorldIT;
uniform highp mat4 matWorldT;
uniform highp mat4 matViewIT;

uniform mediump vec3 Light0Direction;
uniform mediump vec3 Light1Direction;

uniform lowp vec4 Light0Ambientcolor;
uniform lowp vec4 Light1Ambientcolor;

#if defined(SKINNED) && !defined(DCC_MAX)
	uniform highp mat4 BoneMatrices[48];
	uniform highp vec4 WeightMask;
	attribute mediump vec4 SkinWeights;
	attribute mediump vec4 SkinIndices;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

//This shader will not work properly on an unskinned mesh, as we assume that the attributes are already in World Space.
void main()
{
#if defined(SKINNED) && !defined(DCC_MAX)
    highp vec4 weights = SkinWeights * WeightMask;
    highp vec4 netPosition = vec4(0.0, 0.0, 0.0, 0.0);
    highp vec4 netNormal = vec4(0.0, 0.0, 0.0, 0.0);
	highp vec4 netTangent = vec4(0.0, 0.0, 0.0, 0.0);
	highp vec4 netBinormal = vec4(0.0, 0.0, 0.0, 0.0);

	vec4 indices = SkinIndices;

    for (int i = 0; i < 4; i++)
    {
        int index = int(indices[i]);
        netPosition += weights[i] * (BoneMatrices[index] * vec4(Vertex.xyz, 1));
        netNormal += weights[i] * (BoneMatrices[index] * vec4(Normal.xyz, 0));
        netTangent += weights[i] * (BoneMatrices[index] * vec4(Tangent.xyz, 0)); 
        netBinormal += weights[i] * (BoneMatrices[index] * vec4(Binormal.xyz, 0));     
    }
  
 	gl_Position = matWorldViewProjection * netPosition;   
    
   	mediump vec3 eye = normalize(eyeposition - netPosition.xyz);
	
	mediump mat3 objTangentXf; 

	objTangentXf = mat3(netTangent.x, netBinormal.x, netNormal.x,
        	                netTangent.y, netBinormal.y, netNormal.y,
            	            netTangent.z, netBinormal.z, netNormal.z);
#else
	// Vertex Position output
	gl_Position = matWorldViewProjection * Vertex;

	mediump vec3 eye = normalize(eyeposition - Vertex.xyz);
	
	mediump mat3 objTangentXf; //build object to tangent space transform matrix 
	#ifdef DCC_MAX
		objTangentXf = mat3(Binormal.x, -Tangent.x, Normal.x,
        	                Binormal.y, -Tangent.y, Normal.y,
            	            Binormal.z, -Tangent.z, Normal.z);
	#else
		objTangentXf = mat3(Tangent.x, Binormal.x, Normal.x,
        	                Tangent.y, Binormal.y, Normal.y,
            	            Tangent.z, Binormal.z, Normal.z);
	#endif
#endif

	vVP[0] = objTangentXf * Light0Direction;
	vVP[1] = objTangentXf * Light1Direction;
	lowp vec4 Ambient = Light0Ambientcolor + Light1Ambientcolor;
	
	vec4 color = clamp(vec4(Ambient.xyz, 1.0), 0.0, 1.0);
	vColor = color;
		
#ifdef DCC_MAX
	SunOcclusionFactor = max(SunOcclusionFactor, 1.0);
	SunOcclusionFactor = min(SunOcclusionFactor, 0.0);
	vColor += SunOcclusionFactor;	// [ICH] keep SunOcclusionFactor
#endif

	#ifdef DCC_MAX
		if (vColor.rgb == vec3(0.0))
    		vColor = vec4(1.0);
    #endif
	
	//only calculate specular using light 0.
	vHalfVector = objTangentXf * normalize(Light0Direction + eye);

#ifdef USE_BACK_LIGHT
	BacklightHalfVector = objTangentXf * normalize(eye - Light0Direction);
#endif // USE_BACK_LIGHT

	vTexCoord0 = texcoord0;

#ifdef FG
	#if defined(SKINNED) && !defined(DCC_MAX)
		highp float fogDistance = dot(matWorldViewT[2], netPosition);
	#else
		highp float fogDistance = dot(matWorldViewT[2], Vertex); //length(fogVector);
	#endif
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
} // main end
