#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define SKINNED
//#define FG

attribute highp vec4 Vertex DCC(: POSITION);

attribute highp vec3 Normal DCC(: NORMAL);

attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

varying lowp vec4 vColor DCC(: COLOR0);

uniform lowp vec4 light0ambientcolor;

#if defined(SKINNED) && !defined(DCC_MAX)
	uniform highp mat4 BoneMatrices[48];
	uniform highp vec4 WeightMask;
	attribute mediump vec4 SkinWeights;
	attribute mediump vec4 SkinIndices;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

uniform highp mat4 matWorldViewProjection;
uniform highp mat4 matWorldIT;

uniform mediump vec4 irdcAr; 
uniform mediump vec4 irdcAg;
uniform mediump vec4 irdcAb;
uniform mediump vec4 irdcBr;
uniform mediump vec4 irdcBg;
uniform mediump vec4 irdcBb;
uniform mediump vec3 irdcC;

uniform mediump vec3 IrrAmbientDiffuseMult;

void main()
{
#if defined(SKINNED) && !defined(DCC_MAX)
    highp vec4 weights = SkinWeights * WeightMask;
    highp vec4 netPosition = vec4(0.0, 0.0, 0.0, 0.0);
    highp vec4 netNormal = vec4(0.0, 0.0, 0.0, 0.0);

	vec4 indices = SkinIndices;

    for (int i = 0; i < 4; i++)
    {
        int index = int(indices[i]);
        netPosition += weights[i] * (BoneMatrices[index] * vec4(Vertex.xyz, 1));
        netNormal += weights[i] * (BoneMatrices[index] * vec4(Normal.xyz, 0));    
    }
 
	// Vertex Position output
	gl_Position = matWorldViewProjection * netPosition;

	//highp vec3 transformedNormal = (matWorldIT * vec4(netNormal.xyz, 0.0)).xyz;
	highp vec3 transformedNormal = netNormal.xyz;
	
#else
	// Vertex Position output
	gl_Position = matWorldViewProjection * Vertex;

	//highp vec3 transformedNormal = (matWorldIT * vec4(Normal, 0.0)).xyz;
	highp vec3 transformedNormal = Normal.xyz;	
#endif
	
	vec4 normal = vec4(transformedNormal, 1.0);
	
	normal.xyz = normalize(normal.xyz);
	normal.yz = normal.zy;
	normal.y *= -1.0;	
 
	mediump vec3 x1, x2, x3;
    
    // Linear + constant polynomial terms
	x1.r = dot(irdcAr,normal);
    x1.g = dot(irdcAg,normal);
    x1.b = dot(irdcAb,normal);

    // 4 of the quadratic polynomials
    mediump vec4 vB = normal.xyzz * normal.yzzx;   
    x2.r = dot(irdcBr,vB);
    x2.g = dot(irdcBg,vB);
    x2.b = dot(irdcBb,vB);

    // Final quadratic polynomial
    mediump float vC = normal.x*normal.x - normal.y*normal.y;
    x3 = irdcC.rgb * vC;    

   	lowp vec3 diffuse = x1 + x2 + x3;
    
    diffuse = max(vec3(0.0, 0.0, 0.0), diffuse);

    #ifdef DCC_MAX
    	IrrAmbientDiffuseMult = min(vec3(0.0, 0.0, 0.0), IrrAmbientDiffuseMult);
    	IrrAmbientDiffuseMult = max(vec3(0.0, 0.0, 0.0), IrrAmbientDiffuseMult);
	    vColor = vec4(1.0);
	    vColor.xyz += IrrAmbientDiffuseMult;
    #else
    	lowp vec3 finalAmbientDiffuseColor =  light0ambientcolor.xyz * IrrAmbientDiffuseMult.x + diffuse * IrrAmbientDiffuseMult.y;
    
	    vColor = vec4(finalAmbientDiffuseColor, 1.0);
    #endif
    
	vTexCoord0 = texcoord0;

/////////////////////////
//Fog factor output	   //
/////////////////////////
#ifdef FG
	highp float fogDistance = dot(matWorldViewT[2], Vertex);
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
} // main end
