#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define SKINNED
//#define FG

attribute highp vec4 Vertex DCC(: POSITION);

attribute highp vec3 Normal DCC(: NORMAL);

attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);

#if defined(SKINNED) && !defined(DCC_MAX)
	uniform highp mat4 BoneMatrices[48];
	uniform highp vec4 WeightMask;
	attribute mediump vec4 SkinWeights;
	attribute mediump vec4 SkinIndices;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

varying lowp vec4 vColor DCC(: COLOR0);

uniform highp vec3 Light0DirectionOS;
uniform lowp vec4 Light0AmbientColor;
uniform lowp vec4 Light0DiffuseColor;

uniform highp vec3 Light1DirectionOS;
uniform lowp vec4 Light1DiffuseColor;

uniform lowp float SunOcclusionFactor;

uniform highp mat4 matWorldViewProjection;

void main()
{
#if defined(SKINNED) && !defined(DCC_MAX)
    highp vec4 weights = SkinWeights * WeightMask;
    highp vec4 netPosition = vec4(0.0, 0.0, 0.0, 0.0);
    highp vec4 netNormal = vec4(0.0, 0.0, 0.0, 0.0);

	vec4 indices = SkinIndices;

    for (int i = 0; i < 4; i++)
    {
        int index = int(indices[i]);
        netPosition += weights[i] * (BoneMatrices[index] * vec4(Vertex.xyz, 1));
        netNormal += weights[i] * (BoneMatrices[index] * vec4(Normal.xyz, 0));    
    }
 
  	gl_Position = matWorldViewProjection * netPosition;

    lowp float nDotVP0 = max(0.0, dot(netNormal.xyz, Light0DirectionOS));
    lowp float nDotVP1 = max(0.0, dot(netNormal.xyz, Light1DirectionOS));  
    lowp vec3 Diffuse = Light0DiffuseColor.rgb * nDotVP0 + Light1DiffuseColor.rgb * nDotVP1;
#else
	gl_Position = matWorldViewProjection * Vertex;

    lowp float nDotVP = max(0.0, dot(Normal, Light0DirectionOS)); 
    lowp vec3 Diffuse = Light0DiffuseColor.rgb * nDotVP;
    nDotVP = max(0.0, dot(Normal, Light1DirectionOS)); 
    Diffuse +=  Light1DiffuseColor.rgb * nDotVP;
#endif
	
    lowp vec4 color = vec4(Light0AmbientColor.rgb + Diffuse.xyz * SunOcclusionFactor, 1.0);

	vColor = color;
	
#ifdef DCC_MAX
	if (vColor.rgb == vec3(0.0))
 		vColor = vec4(1.0);
#endif

	vTexCoord0 = texcoord0;
	
#ifdef FG
#if defined(SKINNED) && !defined(DCC_MAX)
	highp float fogDistance = dot(matWorldViewT[2], netPosition);
#else
	highp float fogDistance = dot(matWorldViewT[2], Vertex);
#endif
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
}