//#define ADDITIVE
//#define DIFFUSE_TINT
//#define TONED


uniform sampler2D 	ColorMapSampler;
#ifdef SPLIT_ALPHA
uniform sampler2D 	ColorMapSampler_alpha;
#endif

uniform sampler2D 	MasksSampler;
#ifdef SPLIT_ALPHA
uniform sampler2D 	MasksSampler_alpha;
#endif

uniform lowp 	vec4 		DiffuseTint;

// Forcing Tone offset and gain temporarily on all effects.
//#define TONED
#ifdef TONED
uniform lowp 	vec3 		ToneGain;
uniform lowp 	vec3 		ToneOffset;
#endif

// == Interpolators 	== //
varying   	mediump 	vec2 vCoord0;
varying		mediump		vec2 vOffsetMapCoord0;
varying		highp		vec2 vOffsetMapCoord1;
varying		highp		vec2 vOffsetMapCoord2;
varying		mediump		vec2 vGridUV;
varying		highp		vec2 vStarsUV;
varying		mediump		float vTimer;
varying   	lowp		vec4 vColor;

void main()
{
	//lowp vec4 OffsetMap0 = texture2D(MasksSampler, vOffsetMapCoord0);
	highp float OffsetMap0 = texture2D(MasksSampler, vOffsetMapCoord0).r;
	
		
	highp vec2 OffsetMap1 = vOffsetMapCoord0*0.5;
	OffsetMap1 += OffsetMap0*0.1;	
	OffsetMap1 += vOffsetMapCoord1;
	
	//Stars Sampling
	lowp vec4 ColorSample = texture2D(ColorMapSampler, fract(OffsetMap1)*fract(vStarsUV));
	lowp float Stars = ColorSample.a; 			
	//	
	
	highp vec4 OffsetMap2 = texture2D(MasksSampler, OffsetMap1);
	OffsetMap1 = vOffsetMapCoord2;									
	OffsetMap1 += OffsetMap2.r * 0.5;
	
	lowp vec4 ColorMap = texture2D(ColorMapSampler, OffsetMap1);			
	lowp vec4 Highlight = ColorSample * ColorMap.a;
	lowp float Grid = (texture2D(MasksSampler, vGridUV)).g ;

	//Transition Gridmap
	lowp float GridMap = texture2D(MasksSampler, vec2(OffsetMap2.g,OffsetMap2.g)).r; 	//Transition Gridmap		
	//
	
	//Stars Transform
	Stars *= OffsetMap2.r*6.0;
	//

	// Using no gridmap version.
	//lowp vec4 final	=  Grid * mix((ColorMap + Highlight), ColorMap*(Stars+GridMap * 0.00001), vTimer );
	lowp vec4 final	=  Grid * mix((ColorMap + Highlight), ColorMap*(Stars), vTimer );				
	
	
	// Diffuse color should always be white. This is just for compatibility reasons.
	// For color tinting use DiffuseTint;

 #ifdef DIFFUSE_TINT
 	final	*= DiffuseTint;
 #endif
	

#ifdef TONED
	final.rgb = final.rgb * ToneGain + ToneOffset * final.a;
#endif


	gl_FragColor = final;//vec4(vOffsetMapCoord0.x, vOffsetMapCoord0.y, 0.0, 1.0);
}