uniform lowp mat4 ColorMatrix; // Android

//#define ADDITIVE
//#define DIFFUSE_TINT
//#define TONED
#define SPLIT_ALPHA // Android

uniform sampler2D 	DiffuseSampler;
#ifdef SPLIT_ALPHA
uniform sampler2D 	DiffuseSampler_alpha;
#endif

uniform sampler2D 	MaskSampler;
#ifdef SPLIT_ALPHA
uniform sampler2D 	MaskSampler_alpha;
#endif

uniform lowp 	vec4 		DiffuseTint;

// Forcing Tone offset and gain temporarily on all effects.
//#define TONED
#ifdef TONED
uniform lowp 	vec3 		ToneGain;
uniform lowp 	vec3 		ToneOffset;
#endif

// == Interpolators 	== //
varying   	mediump 	vec2 vCoord0;
varying		highp		vec2 vCoord1;
varying   	lowp		vec4 vColor;

void main()
{
	lowp vec4 diffuseMap = texture2D(DiffuseSampler, vCoord0);
#ifdef SPLIT_ALPHA
	diffuseMap.a = texture2D(DiffuseSampler_alpha, vCoord0).g;
#endif

	// Mask sampler
	lowp vec4 maskMap = texture2D(MaskSampler, vCoord1);
#ifdef SPLIT_ALPHA
	maskMap.a = texture2D(MaskSampler_alpha, vCoord1).g;
#endif	

	// assign the diffuseMap as the final texture color
	lowp vec4 final	= diffuseMap * maskMap;

	// Diffuse color should always be white. This is just for compatibility reasons.
	// For color tinting use DiffuseTint;

    final	*= (vColor);
    
 #ifdef DIFFUSE_TINT
 	final	*= DiffuseTint;
 #endif
	

#ifdef TONED
	final.rgb = final.rgb * ToneGain + ToneOffset * final.a;
#endif

//	final = vec4(1,1,1,1);

	gl_FragColor = final;
}