//#define DIFFUSE			// Diffuse texture.
//#define DIFFUSE_TINT	 	// Multiplier for colors. Can be used for increasing final color values.
//#define ALPHA				// For using a seperate texture for alpha, ignoring the alpha of the diff texture.
//#define ALPHA_TEST		// For alpha testing.
//#define PREMULTIPLIED 	// The expected format is premultiplied alpha blending.
//#define UV				// Allows using custom UV Animation offsets if DIFFUSE and/or ALPHA were defined.

//#define SPRITESHEET		// This material will expect a sprite sheet animation.
//#define FOURFRAMES		// Allows the use of four frames per pixel (one frame per channel).
//#define FOURFRAMESLERP	// Same as FOURFRAMES except that the frames can be lerped between them. 
							// This way some frames are lost as the alpha channel needs have the next sheet's red channel (duplicated)
//#define TONED				// ToneGain and Offset
#define SPLIT_ALPHA
// === SHADER UNIFORMS === //
#ifdef DIFFUSE
uniform lowp 	sampler2D	DiffuseSampler;
#	ifdef SPLIT_ALPHA
uniform lowp 	sampler2D	DiffuseSampler_alpha;
#	endif
#endif

#ifdef ALPHA
uniform lowp	sampler2D	AlphaSampler;
#endif

#ifdef DIFFUSE_TINT
uniform lowp 	vec4 		DiffuseTint;
#endif

#ifdef DIFFUSE_COLOR
uniform lowp	vec4		DiffuseColor;
#endif

// Forcing Tone offset and gain temporarily on all effects.
#define TONED
#ifdef TONED
uniform lowp 	vec3 		ToneGain;
uniform lowp 	vec3 		ToneOffset;
#endif

// ---


// === INTERPOLATORS === //

varying	mediump vec2 		vCoord0;

#ifdef ALPHA
varying mediump	vec2		vCoord1;
#endif

varying lowp	vec4 		vColor;				// Will always be available for particle systems.

#ifdef ALPHA_TEST
uniform lowp	float 		alphaThreshold;
#endif

#if defined FOURFRAMES || defined FOURFRAMESLERP
varying lowp	vec4		vFrameSel;
#endif



// ---

void main()
{
	lowp vec4 color			= vec4(1,1,1,1);

#ifdef DIFFUSE
	color = texture2D(DiffuseSampler, vCoord0);
#	if defined SPLIT_ALPHA && !defined ALPHA
	color.a = texture2D(DiffuseSampler_alpha, vCoord0).g;
#	endif
#endif

#ifdef ALPHA
	color.a = texture2D(AlphaSampler, vCoord1).r;
#endif

#if defined FOURFRAMES || defined FOURFRAMESLERP
	color = vec4(dot(color, vFrameSel));
#endif

	color *= vColor;							// Vertex color will always be there.
	
#ifdef DIFFUSE_TINT
	color *= DiffuseTint;
#endif

#ifdef ALPHA_TEST
	if(color.a < alphaThreshold)
		discard;
#endif

#ifdef TONED
#	ifdef PREMULTIPLIED
	color.rgb = color.rgb * ToneGain + ToneOffset * color.a;
#	else
	color.rgb = color.rgb * ToneGain; + ToneOffset * color.a;
#	endif
#endif
	
	gl_FragColor = color;// * 0.0001 + vec4(0.0, vCoord0.y, 0.0, 1.0);

}