//#define DIFFUSE			// Diffuse texture.
//#define DIFFUSE_TINT	 	// Multiplier for colors. Can be used for increasing final color values.
//#define ALPHA				// For using a seperate texture for alpha, ignoring the alpha of the diff texture.
//#define ALPHA_TEST		// For alpha testing.
//#define PREMULTIPLIED 	// The expected format is premultiplied alpha blending.
//#define UV				// Allows using custom UV Animation offsets if DIFFUSE and/or ALPHA were defined.

//#define SPRITESHEET		// This material will expect a sprite sheet animation.
//#define FOURFRAMES		// Allows the use of four frames per pixel (one frame per channel).
//#define FOURFRAMESLERP	// Same as FOURFRAMES except that the frames can be lerped between them. 
							// This way some frames are lost as the alpha channel needs have the next sheet's red channel (duplicated)
//#define AL0
//#define AL1
//#define DIFFUSE_COLOR		// Force the use of a "diffuse color" uniform variable for FXEditor diffuse color animation compatibility.

//#define SS				// Screen-Space. Input vertices will be treated in screen-space, hence world transformations will not be taken into account.

//#define TONED

// === DEFINES THAT NEED TO BE ENFORCED === //
// If we have defined FOURFRAMES (or the lerped one) we for sure need sprite sheet
#if defined FOURFRAMES || defined FOURFRAMESLERP
	#ifndef SPRITESHEET
		#define SPRITESHEET
	#endif
#endif

#if defined(FOURFRAMES) && defined(FOURFRAMESLERP)
//#error "FOURFRAMES and FOURFRAMESLERP at the same time."
#endif

// === VERTEX ATTRIBUTES === //
attribute 	highp   	vec4 Vertex;
attribute 	mediump 	vec2 TexCoord0;
attribute 	lowp		vec4 Color;

// ---


// === SHADER UNIFORMS === //
uniform   	highp		mat4 WorldViewProjectionMatrix;

#ifdef UV
uniform 	mediump		vec2 UVOffset;
#	ifdef ALPHA
	uniform 	mediump 	vec2 UVAOffset;
#	endif
#endif

#ifdef SPRITESHEET
uniform 	mediump		vec2 ColumnsRows;
#endif

// ---

// === INTERPOLATORS === //
varying   	lowp		vec4 vColor;
varying   	mediump 	vec2 vCoord0;

#ifdef ALPHA
varying	  	mediump		vec2 vCoord1;
#endif

#if defined FOURFRAMES || defined FOURFRAMESLERP
varying 	mediump 	vec4 vFrameSel;								// #if defined FOURFRAMES (or the lerped) For selecting which channel to use on the frame selection.
#endif

#ifdef SS
uniform		highp		vec2 viewportsize;					// For keeping aspect ratio.
#endif

#ifdef AL0
// LIGHTING: AL0
uniform		lowp		vec3 light0AmbientColor;
#endif

#ifdef AL1
// LIGHTING: AL1
uniform		lowp		vec3 light1AmbientColor;
#endif

// ---

// Program.
void main()
{
	vColor         = Color;
	vCoord0        = TexCoord0;

#ifdef ALPHA
	vCoord1		   = TexCoord0;
#endif

#if defined FOURFRAMES || defined FOURFRAMESLERP
	mediump float frame		= (TexCoord0.x * ColumnsRows.x);
	mediump float iframe	= floor(frame);							// Frame number of the spritesheet.
	mediump float lerp		= frame - iframe;						// The lerp mount inside the a single frame. (i.e. fract).

#	if defined FOURFRAMESLERP
	mediump float lerpQuad	= lerp * 3.0;							// Lerp scaled by three (since there are four (from 0 to 3) pictures in a single frame).
#	elif defined FOURFRAMES
	mediump float lerpQuad 	= lerp * 4.0;
#	endif
	mediump vec4  frameSel	= vec4(lerpQuad) + vec4(1, 0, -1, -2);	// Frame selection depending on the timequad.
	frameSel				= clamp(frameSel, 0.0, 1.0);
	frameSel		   	   -= vec4(frameSel.yzw, 0.0);
	vFrameSel	 			= frameSel;

	vCoord0.x				= iframe / ColumnsRows.x;
#	ifdef ALPHA
	vCoord1.x				= iframe / ColumnsRows.x;
#	endif

#endif

#ifdef UV
	vCoord0		  += UVOffset;
#	ifdef ALPHA
	vCoord1		  += UVAOffset;
#	endif
#endif
	

	
#ifdef DIFFUSE_COLOR
	vColor		  *= DiffuseColor;									// For compatibility reasons only with old FX Particles Runtimes.
#endif

#ifndef SS	
	gl_Position    = WorldViewProjectionMatrix * Vertex;
#else
	float viewportRatio = viewportsize.y / viewportsize.x;
	vec4 position		= Vertex;
	position.x		   *= viewportRatio;
	gl_Position			= position;
#endif
}







