//#define DIFFUSE_TINT
//#define ALPHA_CHOKE		// Choking technique of the alpha. For 'eating up the texture' effect.

//#define PREMULTIPLIED		// Use to hint that the shader will be used along premultiplied alpha blending states. (ONE OneMinusSrcAlpha)
//#define ADDITIVE			// Use to hint that the shader will be used along with additive blending states (ONE ONE)
//#define COLORDODGE		// Use to hint that the shader will use Color Dodge blend: Hardware blend states needed: finalCOlor = src * dst + src * dst.
//#define ALPHA				// Use to hint that the shader will use normal alpha blending states (SrcAlpha OneMinusSrcAlpha) 
//#define SCREENBLEND		// Signal that the shader is gonna be using renderstate to simulate 'Screen blend mode'.
//#define MUL				// To allow the shader to work properly with multiply blending modes.

//#define DL0				// Define to use Diffuse Lighting. 
//#define UV				// Define to use uv animation.
//#define TONED				// Define to allow the particles to be toned by ToneGain and ToneOffset.
//#define DIFFSS			// Screen texture sampling.
//#define CUSTOMVXATTRIBUTE	// Use alpha choke but with custom attributes..

#define SPLIT_ALPHA

#ifndef DCC_MAX
// needed to support texture atlassing
//#define DiffuseSampler DiffuseSampler_glitchprops_semtexture_tcTexCoord0
#	ifdef SPLIT_ALPHA
//#define DiffuseSampler_alpha DiffuseSampler_alpha_glitchprops_semtexture_tcTexCoord0
#	endif
#endif

uniform lowp 	sampler2D	DiffuseSampler;
#ifdef SPLIT_ALPHA
uniform lowp 	sampler2D	DiffuseSampler_alpha;
#endif

#ifdef DIFFSS

uniform lowp	sampler2D	DiffuseScreenSampler;
#endif

uniform lowp 	vec4 		DiffuseTint;

// Forcing Tone offset and gain temporarily on all effects.
#define TONED
#ifdef TONED
uniform lowp 	vec3 		ToneGain;
uniform lowp 	vec3 		ToneOffset;
#endif

varying	mediump vec2 		vCoord0;

#ifdef DIFFSS
varying mediump vec2		vCoord1;
#endif

#ifdef ALPHA_CHOKE
varying lowp	vec2		vAlphaChoke1;		// ac_value on x, ac_softness on y.
#endif

#ifdef DL0
varying lowp	vec3		vLightDir0;
#endif

varying lowp	vec4 		vColor;

#ifdef ALPHA_TEST
	float alphaThreshold;
#endif

void main()
{
	lowp vec4 diffuseMap = texture2D(DiffuseSampler, vCoord0);
#ifdef SPLIT_ALPHA
	diffuseMap.a = texture2D(DiffuseSampler_alpha, vCoord0).r;
#endif

#ifdef ALPHA_TEST
	if(diffuseMap.a < alphaThreshold)
		discard;
#endif
	
	lowp vec4 final = diffuseMap;
#ifdef DL0
	final 			= vec4(1,1,1, final.a);
	// If using lights, diffuseMap is a normal.
	lowp vec3 n				= (diffuseMap.rgb - 0.5) * 2.0;
	lowp float nDotL		= dot(n, vLightDir0);
	lowp float halfLambert 	= (nDotL * 0.5) + 0.5;
	halfLambert		       *= halfLambert;
	lowp float lighting		= halfLambert;
#endif	
	
	final 		   *= vColor;
#ifdef DIFFUSE_TINT
	final 		   *= DiffuseTint;
#endif

#ifdef DL0
	lowp vec3 lightedColor			= final.rgb * lighting;
	lightedColor				   += mix(final.rgb, vec3(0,0,0), mix(1.0, diffuseMap.a, -nDotL));
	final.rgb						= lightedColor;
#endif

#ifdef ALPHA_CHOKE
//	lowp float crunch 	= smoothstep(vAlphaChoke1.x - vAlphaChoke1.y, vAlphaChoke1.x + vAlphaChoke1.y, final.a);
	
	lowp float crunch 	= mix(vAlphaChoke1.x, vAlphaChoke1.y, diffuseMap.a);
	crunch				= clamp(crunch, 0.0, 1.0);
	crunch		   	   *= crunch;
#	if defined (PREMULTIPLIED) || defined (ADDITIVE)
	final		   *= crunch;
#	else
	final.a			= crunch;
#	endif

#endif

#ifdef DIFFSS
	final		  *= texture2D(DiffuseScreenSampler, vCoord1);
#endif

#ifdef TONED
#	ifdef PREMULTIPLIED
	final.rgb = final.rgb * ToneGain + ToneOffset * final.a;
#	else
	final.rgb = final.rgb * ToneGain; // + ToneOffset * final.a;
#	endif
#endif

#ifdef MUL
	final.rgb = mix(vec3(1.0, 1.0, 1.0), final.rgb, final.a);
#endif

#ifdef COLORDODGE
	lowp vec4 OneMinusFinal 	= 1.0 - clamp(final, 0.0, 1.0);
	final						= 0.5 / OneMinusFinal;
#endif

	gl_FragColor = final;
}