//#define DIFFUSE_COLOR		// Uncomment to ensure compatibility with previous Particles/ColladaVersions (COLLADA version 858)
//#define DEPTH_BIAS
//#define ALPHA_CHOKE		// Choking technique of the alpha. For 'eating up the texture' effect.

//#define PREMULTIPLIED		// Use to hint that the shader will be used along premultiplied alpha blending states. (ONE OneMinusSrcAlpha)
//#define ADDITIVE			// Use to hint that the shader will be used along with additive blending states (ONE ONE)
//#define SCREENBLEND		// Signal that the shader is gonna be using renderstate to simulate 'Screen blend mode'.
//#define MUL
//#define COLORDODGE		// Use to hint that the shader will use Color Dodge blend: Hardware blend states needed: finalCOlor = src * dst + src * dst.
//#define ALPHA				// Use to hint that the shader will use normal alpha blending states (SrcAlpha OneMinusSrcAlpha) 

//#define DL0				// Define to use Diffuse Lighting. 
//#define AL0				// Define to use Light0AmbientColor.
//#define AL1				// Define to use Light1AmbientColor.
//#define UV				// Define to use uv animation.
//#define TONED				// Define to allow the particles to be toned by ToneGain and ToneOffset.
//#define KEEP_UV			// Keeps the original UV. Since particles (when this shader was created) only have single UV animation, sometimes we calculate the UV's (since they are billboards) in the vertex shader and use the UV Animation curves to animate some extra parameters.
//#define SS				// 'Screen-space' positioning. Ignores the view projection matrix.

//#define DIFFSS			// Screen Space texture sampling.
//#define CUSTOMVXATTRIBUTE	// Use alpha choke but with custom attributes.


// == Vertex Attributes == //
attribute 	highp   	vec4 Vertex;
attribute 	mediump 	vec2 TexCoord0;
attribute 	lowp		vec4 Color;

#ifdef CUSTOMVXATTRIBUTE
attribute	mediump		vec4 customVXAttribute;
#endif


// == Uniform variables == //
#ifndef SS
uniform   	highp		mat4 WorldViewProjectionMatrix;
#endif

#ifdef SS
uniform		highp		mat4 World;
uniform		mediump		vec2 ViewportSize;
uniform		highp		mat4 matprojectioncompensation;
#endif

uniform 	mediump 	vec4 DiffuseSampler_scaleoffset;

#ifdef DEPTH_BIAS
uniform		highp		vec3 eyeposos;
uniform		highp		float DepthBias;
uniform		highp		mat4 viewT;
#endif

#ifdef DIFFUSE_COLOR
uniform lowp			vec4 DiffuseColor;
#endif

#if defined(ALPHA_CHOKE) && defined(KEEP_UV)
uniform		mediump		float ac_value;
uniform		mediump		float ac_softness;
#endif

#ifdef DL0
// LIGHTING: DL0
uniform 	highp 		vec3 light0PositionOS;
uniform 	lowp 		vec3 light0Color;
uniform 	highp 		vec3 light0DirOS;
uniform		highp		mat4 worldview;
#endif

#ifdef AL0
// LIGHTING: AL0
uniform		lowp		vec3 light0AmbientColor;
#endif

#ifdef AL1
// LIGHTING: AL1
uniform		lowp		vec3 light1AmbientColor;
#endif


#ifdef UV
uniform		mediump		vec2 UVOffset;
#endif

// == Interpolators 	== //
varying   	mediump 	vec2 vCoord0;

#ifdef DIFFSS
varying		mediump		vec2 vCoord1;
#endif

#ifdef ALPHA_CHOKE
varying		lowp		vec2 vAlphaChoke1;	//Alpha choke variables here. (1 stands forTEXCOORD1)
#endif

#ifdef DL0
varying		mediump		vec3 vLightDir0;	// vLightDir result.
#endif


varying   	lowp		vec4 vColor;

void main()
{

	vec2 tCoord0   = TexCoord0 * DiffuseSampler_scaleoffset.xy + DiffuseSampler_scaleoffset.zw;
#ifdef UV	
	tCoord0       += UVOffset;
#endif
	vCoord0        = tCoord0;
	vColor         = Color;
	
#ifdef AL0
	vColor.rgb	  += light0AmbientColor.rgb; 
#endif

#ifdef AL1
	vColor.rgb	  += light1AmbientColor.rgb;
#endif
	
	vec4 position  		= Vertex;

#ifdef SS
	position			= World * Vertex;
	float viewportRatio = ViewportSize.y / ViewportSize.x;
	position			= position;
	position.x		   *= viewportRatio;
	position.y		   *= -1.0;									// Temporal hack 
#else
	position			= WorldViewProjectionMatrix * Vertex;
#endif	



#ifdef DEPTH_BIAS

	// Try-out 1:
	// vec3 viewVec		= Vertex.xyz - eyeposos;
	// viewVec		= viewVec * -view[2];
	// position			= WorldViewProjectionMatrix * (Vertex + vec4(viewVec * DepthBias * 0.1, 0));
	
	// Original:
	vec3 viewVec		= normalize(Vertex.xyz - eyeposos);
	vec3 forwardVec		= -viewT[2].xyz;
	float projAdj		= 1.0 / dot(viewVec, forwardVec);

#ifdef SS
	position			= World * (Vertex + vec4(viewVec * (DepthBias * projAdj), 0));
	float viewportRatio = ViewportSize.y / ViewportSize.x;
	position.x		   *= viewportRatio;
	position			= matprojectioncompensation * position;
#else
	position			= WorldViewProjectionMatrix * (Vertex + vec4(viewVec * (DepthBias * projAdj), 0));
#endif	
	
	
#endif

#ifdef ALPHA_CHOKE

#	ifndef CUSTOMVXATTRIBUTE
		vCoord0			= floor(TexCoord0);
		vAlphaChoke1	= TexCoord0;
		// .x -> ac_value ; .y -> ac_softness;
		vAlphaChoke1	= vec2(vAlphaChoke1.x - vAlphaChoke1.y, vAlphaChoke1.x + vAlphaChoke1.y);

#		ifdef KEEP_UV
			vCoord0			= TexCoord0;
			vAlphaChoke1	= vec2(ac_value - ac_softness, ac_value + ac_softness);
#		endif
#	else
		vCoord0			= TexCoord0;
		vAlphaChoke1	= customVXAttribute.xy;
		vAlphaChoke1	= vec2(vAlphaChoke1.x - vAlphaChoke1.y, vAlphaChoke1.x + vAlphaChoke1.y);
#	endif

#endif

#ifdef DIFFSS
	mediump vec2 screenCoord = position.xy / position.w;
	vCoord1 =  screenCoord * 0.5 + 0.5;
#endif

	
#ifdef DIFFUSE_COLOR
	vColor		  *= DiffuseColor;
#endif

#ifdef DL0
	mediump vec3 lightDirOS		= normalize(light0PositionOS - Vertex.xyz);
	vLightDir0					= mat3(worldview) * lightDirOS;
#endif

//	position.z		= clamp(position.z, -1.0, 1.0);

	gl_Position    	= position;
}
