//#define SPLIT_ALPHA
//#define DIFFUSE_TINT
//#define ALPHA_TEST
//#define DEPTH_BIAS
//#define PREMULTIPLIED
//#define ADDITIVE
//#define ALPHABLEND
//#define UV
//#define TONED						// Watch out as this is sometimes forced down below.
//#define MASK
//#define MASK2
//#define COLORBYDIR					// Define this to have access to back-front coloring depending on angle between camera an tangent.
//#define SPEEDBYVC						// Vertex colors multiply the velocity of the cars. R for UVOffset, G for Mask1, B for Mask2
//#define DEBUGVC
//#define FG
//#define CAMERAFADE

#define SPLIT_ALPHA

#ifndef DCC_MAX
// needed to support texture atlassing
//#define DiffuseSampler DiffuseSampler_glitchprops_semtexture_tcTexCoord0
#	ifdef SPLIT_ALPHA
//#define DiffuseSampler_alpha DiffuseSampler_alpha_glitchprops_semtexture_tcTexCoord0
uniform	sampler2D DiffuseSampler_alpha;
#	endif
#endif

// == Uniform Variables == //
uniform	sampler2D DiffuseSampler;

#ifdef MASK
uniform	sampler2D MaskSampler;
#endif
#ifdef MASK2
uniform sampler2D MaskSampler2;
#endif

#ifdef DIFFUSE_TINT
uniform 	lowp		vec4 DiffuseTint;									// To pump up colors, usually when tintint a black and white texture.
#endif

#ifdef COLORBYDIR
uniform		mediump		vec4 BackColorTint;					// Back/Front coloring depending on angle between camera and tangent.
uniform		mediump		vec4 FrontColorTint;
#endif

// Forcing Tone offset and gain temporarily on all effects.
#define TONED
#ifdef TONED
uniform lowp 	vec3 		ToneGain;
uniform lowp 	vec3 		ToneOffset;
#endif

// == Interpolators 	== //
varying   	highp 	vec2 vCoord0;		// Diffuse texcoords

#ifdef MASK
varying		highp	vec2 vCoord1;		// Mask texcoords
#endif

#ifdef MASK2
varying		highp	vec2 vCoord2;		// Mask2 texCoords;
#endif

#ifdef COLORBYDIR
varying		lowp	float vDirLerp;		// #if defined COLORBYDIR "Car traffic direction from camera"
#endif

#ifdef ALPHA_CHOKE
varying 	lowp	vec2 vAlphaChoke1;		// ac_value on x, ac_softness on y.
#endif

#ifdef DEBUGVC
varying 	lowp	vec4 vColor;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform lowp vec4 fogcolor;   
#endif

#ifdef CAMERAFADE
varying		lowp	float vCamFade;
#endif

void main()
{
	lowp vec4 diffuseMap = texture2D(DiffuseSampler, vCoord0);
#ifdef SPLIT_ALPHA
	diffuseMap.a 	= texture2D(DiffuseSampler_alpha, vCoord0).r;
#endif
	
	// Start accumulating values over this final variable depending on defines.
	lowp vec4 final = diffuseMap;

#ifdef MASK
	final		   *= texture2D(MaskSampler, vCoord1).r;
#endif

#ifdef MASK2
	final		   *= texture2D(MaskSampler2, vCoord2).r;
#endif

	
#ifdef DIFFUSE_TINT
	final 		   *= DiffuseTint;
#endif

#ifdef ALPHA_CHOKE
//	lowp float crunch 	= smoothstep(vAlphaChoke1.x - vAlphaChoke1.y, vAlphaChoke1.x + vAlphaChoke1.y, final.a);
	
	lowp float crunch 	= mix(vAlphaChoke1.x, vAlphaChoke1.y, diffuseMap.a);
	crunch				= clamp(crunch, 0.0, 1.0);
	crunch		   	   *= crunch;
#	if defined (PREMULTIPLIED) || defined (ADDITIVE)
	final		   *= crunch;
#	else
	final.a			= crunch;
#	endif

#endif

#ifdef TONED
#	ifdef PREMULTIPLIED
	final.rgb = final.rgb * ToneGain + ToneOffset * final.a;
#	else
	final.rgb = final.rgb * ToneGain;	// + ToneOffset * final.a;
#	endif
#endif

#ifdef COLORBYDIR
//	mediump float tangentLerp = clamp(vDirLerp, -1.0, 1.0);
	mediump float tangentLerp = vDirLerp;
	mediump vec4 colorByDir = mix(BackColorTint, FrontColorTint, tangentLerp);
	final = mix(final * colorByDir, final, final.a);
#endif

#ifdef FG
	final.xyz = mix(final.xyz, fogcolor.xyz, vFogFactor);
#endif

#ifdef CAMERAFADE
	final *= vCamFade;
#endif

	gl_FragColor = final;
	
#ifdef DEBUGVC
	gl_FragColor = final * 0.0001 + vColor;
#endif
}