//#define SPLIT_ALPHA
//#define DIFFUSE_TINT
//#define ALPHA_TEST
//#define DEPTH_BIAS
//#define PREMULTIPLIED
//#define ADDITIVE
//#define ALPHABLEND
//#define UV
//#define TONED
//#define MASK
//#define MASK2
//#define COLORBYDIR					// Define this to have access to back-front coloring depending on angle between camera an tangent.
//#define SPEEDBYVC						// Vertex colors multiply the velocity of the cars. R for UVOffset, G for Mask1, B for Mask2
//#define DEBUGVC
//#define FG
//#define CAMERAFADE
//#define GLOBALCAMERAFADE				// Define to use a global parameter for the near far traffic fade. (TrafficFadeStartEnd parameter)					

// == Vertex Attributes == //
attribute 	highp   	vec4 Vertex;

attribute 	mediump 	vec2 TexCoord0;
#ifdef MASK
attribute	mediump		vec2 TexCoord1;
#endif
#ifdef MASK2
attribute	mediump		vec2 TexCoord2;
#endif
#ifdef SPEEDBYVC
attribute 	lowp		vec4 Color;
#endif

#ifdef COLORBYDIR
attribute	mediump		vec3 Normal;
attribute	mediump		vec3 Binormal;
attribute	mediump		vec3 Tangent;
#endif


// == Uniform variables == //
uniform   	highp		mat4 WorldViewProjectionMatrix;

uniform	sampler2D DiffuseSampler;

#ifdef MASK
uniform	sampler2D MaskSampler;
#endif
#ifdef MASK2
uniform sampler2D MaskSampler2;
#endif

#ifdef DIFFUSE_TINT
uniform 	lowp		vec4 DiffuseTint;									// To pump up colors, usually when tintint a black and white texture.
#endif

// UV offsets
#ifdef UV
uniform		mediump		vec2 UVOffset;
uniform		mediump		vec2 UVOffsetSpeed;

// UV Mask Offsets
uniform		mediump		vec2 UVMaskOffset;
uniform		mediump		vec2 UVMaskOffsetSpeed;

uniform		mediump		vec2 UVMaskOffset2;
uniform		mediump		vec2 UVMaskOffsetSpeed2;
#endif

// Timing
uniform		highp		float MyGlobalTime;
/*uniform		highp		float time;*/

#if defined COLORBYDIR

uniform		mediump		float TangentSlope;
uniform		highp		vec3 camerapositionos;
#endif

//#ifdef DEPTH_BIAS
//uniform		highp		vec3 eyeposos;
//uniform		highp		float DepthBias;
//#endif

// Camera Distance Fade
#ifdef CAMERAFADE
uniform		highp		vec2 CameraFadeStartEnd;
//uniform		highp		vec3 cameraposition;
uniform		highp		mat4 worldview;
#	ifdef GLOBALCAMERAFADE
	uniform		highp	vec2 TrafficFadeStartEnd;
#	endif
#endif

// == Interpolators 	== //
varying   	highp 	vec2 vCoord0;		// Diffuse texcoords

#ifdef MASK
varying		highp	vec2 vCoord1;		// Mask texcoords
#endif

#ifdef MASK2
varying		highp	vec2 vCoord2;		// Mask2 texCoords;
#endif

#ifdef COLORBYDIR
varying		lowp	float vDirLerp;		// #if defined COLORBYDIR "Car traffic direction from camera"
#endif

#ifdef DEBUGVC
varying		lowp	vec4 vColor;
#endif

#ifdef FG
varying 	lowp 	float vFogFactor;
uniform 	highp	vec2 fogstartend;
uniform 	highp 	mat4 matWorldViewT;
#endif

#ifdef CAMERAFADE
varying		lowp	float vCamFade;
#endif
void main()
{

	vec2 tCoord0   	= TexCoord0;
#ifdef MASK
	vec2 tCoord1	= TexCoord1;
#endif
#ifdef MASK2
	vec2 tCoord2	= TexCoord2;
#endif

	
#ifdef UV	
	tCoord0       	= TexCoord0 + UVOffset + UVOffsetSpeed * MyGlobalTime;
#	ifdef SPEEDBYVC
		tCoord0       	= TexCoord0 + UVOffset + UVOffsetSpeed * MyGlobalTime  * Color.r;
#	endif
	
#	ifdef MASK
	tCoord1       	= TexCoord1 + UVMaskOffsetSpeed * MyGlobalTime; 
#		ifdef SPEEDBYVC
			tCoord1       	= TexCoord1 + UVMaskOffset + UVMaskOffsetSpeed * MyGlobalTime  * Color.g;
#		endif	
#	endif
#	ifdef MASK2
	tCoord2       	= TexCoord2 + UVMaskOffset2 + UVMaskOffsetSpeed2 * MyGlobalTime;
#		ifdef SPEEDBYVC
			tCoord2       	= TexCoord2 + UVMaskOffset2 + UVMaskOffsetSpeed2 * MyGlobalTime  * Color.b;
#		endif	
#	endif
#endif

	vCoord0        	= tCoord0;
#ifdef MASK
	vCoord1			= tCoord1; 
#endif
#ifdef MASK2
	vCoord2			= tCoord2;
#endif	

	vec4 position  =  	WorldViewProjectionMatrix * Vertex;
#ifdef DEPTH_BIAS
	vec3 viewVec		= normalize(Vertex.xyz - eyeposos);
	position			= WorldViewProjectionMatrix * (Vertex + vec4(viewVec * DepthBias, 0));
#endif

#ifdef ALPHA_CHOKE
	vCoord0			= floor(TexCoord0);
	vAlphaChoke1	= TexCoord0;
	// .x -> ac_value ; .y -> ac_softness;
	vAlphaChoke1	= vec2(vAlphaChoke1.x - vAlphaChoke1.y, vAlphaChoke1.x + vAlphaChoke1.y);
#endif

#ifdef COLORBYDIR
	mediump mat3 objTangentXf 		= mat3(Tangent.x, Binormal.x, Normal.x,
		    	                			Tangent.y, Binormal.y, Normal.y,
		        	            			Tangent.z, Binormal.z, Normal.z);
 
 	mediump vec3 viewVec 			= normalize(Vertex.xyz - camerapositionos);
 	lowp float tangentDir 			= (viewVec * objTangentXf).x;
 	lowp float dirLerp				= clamp(tangentDir * TangentSlope, -1.0, 1.0);
 	dirLerp							= dirLerp * 0.5 + 0.5;
 	vDirLerp						= dirLerp;
#endif
#ifdef DEBUGVC
	vColor			= Color;
#endif

// CAMERA FADE
#ifdef CAMERAFADE
	float cameraDistance = -(worldview * Vertex).z;//length(fogVector);
	//float cameraDistance = length(position.xyz - cameraposition);
	vCamFade = clamp((cameraDistance - CameraFadeStartEnd.x) / (CameraFadeStartEnd.y - CameraFadeStartEnd.x), 0.0, 1.0);
#	ifdef GLOBALCAMERAFADE
		vCamFade = clamp((cameraDistance - TrafficFadeStartEnd.x) / (TrafficFadeStartEnd.y - TrafficFadeStartEnd.x), 0.0, 1.0);
#	endif
#endif

/////////////////////////
//Fog factor output	   //
/////////////////////////
#ifdef FG
	float fogDistance = dot(matWorldViewT[2], Vertex);//length(fogVector);
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif

	gl_Position    = position;
}