#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

attribute highp vec4 Vertex DCC(: POSITION);

attribute highp vec3 Normal DCC(: NORMAL);

attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

varying lowp vec4 vColor DCC(: COLOR0);

uniform lowp vec4 light0ambientcolor;

uniform lowp float SunOcclusionFactor;

#ifdef UV
	uniform mediump float offsetU;
	uniform mediump float offsetV;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

uniform highp mat4 matWorldViewProjection;
uniform highp mat4 matWorldIT;

uniform mediump vec4 irdcAr; 
uniform mediump vec4 irdcAg;
uniform mediump vec4 irdcAb;
uniform mediump vec4 irdcBr;
uniform mediump vec4 irdcBg;
uniform mediump vec4 irdcBb;
uniform mediump vec3 irdcC;

void main()
{	
	// Vertex Position output
	gl_Position = matWorldViewProjection * Vertex;

	highp vec3 transformedNormal = (matWorldIT * vec4(Normal, 0.0)).xyz;
	vec4 normal = vec4(transformedNormal, 1.0);
	
	normal.xyz = normalize(normal.xyz);
	
	mediump vec3 x1, x2, x3;
    
    // Linear + constant polynomial terms
	x1.r = dot(irdcAr,normal);
    x1.g = dot(irdcAg,normal);
    x1.b = dot(irdcAb,normal);

    // 4 of the quadratic polynomials
    mediump vec4 vB = normal.xyzz * normal.yzzx;   
    x2.r = dot(irdcBr,vB);
    x2.g = dot(irdcBg,vB);
    x2.b = dot(irdcBb,vB);

    // Final quadratic polynomial
    mediump float vC = normal.x*normal.x - normal.y*normal.y;
    x3 = irdcC.rgb * vC;    

   	lowp vec3 diffuse = x1 + x2 + x3;
    
    lowp vec4 Ambient = light0ambientcolor;
    diffuse = max(vec3(0.0, 0.0, 0.0), diffuse);

    #ifdef DCC_MAX
	    vColor = vec4(1.0);
    #else
	    vColor = vec4(Ambient.xyz + diffuse, 1.0);
    #endif
    
	vTexCoord0 = texcoord0;
	
#ifdef UV
	//UV Animations
	vTexCoord0.x += offsetU;
	vTexCoord0.y += offsetV;
#endif

/////////////////////////
//Fog factor output	   //
/////////////////////////
#ifdef FG
	highp float fogDistance = dot(matWorldViewT[2], Vertex);
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
} // main end
