//#define VERTEX_SHADER
//#define FRAGMENT_SHADER

//#define DEBUG_RENDER_SM
//#define RENDER_SM
//#define DEBUG_LIGHT_AND_SHADOW
//#define LIGHT_AND_SHADOW
//#define DEBUG_RENDER_LISPSM
//#define RENDER_LISPSM
//#define DEBUG_LIGHT_AND_LISPSHADOW
//#define LIGHT_AND_LISPSHADOW

//#define DEBUG_LIGHT_AND_SHADOW_HW
//#define LIGHT_AND_SHADOW_HW
//#define DEBUG_LIGHT_AND_LISPSHADOW_HW
//#define LIGHT_AND_LISPSHADOW_HW

// --------------------------------------------
// ----           DEBUG_RENDER_SM          ----
// ----  Fill shadow map with debug color  ----
// --------------------------------------------

#ifdef DEBUG_RENDER_SM

uniform highp mat4 WorldViewProjectionMatrix;

#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

void main()
{
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

#ifdef FRAGMENT_SHADER
void main()
{
	// Write any color coded debug value in color RT
	const lowp vec4 red = vec4(1.0, 0.0, 0.0, 1.0);
	gl_FragColor = red;
}
#endif
// FRAGMENT_SHADER

#endif 
// DEBUG_RENDER_SM

// ------------------------------------------------------------
// ----                   RENDER_SM                        ----
// ----  Fill shadow map with no specific color            ----
// ------------------------------------------------------------
#ifdef RENDER_SM

uniform highp mat4 WorldViewProjectionMatrix;

#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

void main()
{
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

#ifdef FRAGMENT_SHADER
void main()
{
	gl_FragColor = vec4(1.0,0.0,0.0,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// RENDER_SM

// ------------------------------------------------------------
// ----              DEBUG_LIGHT_AND_SHADOW                ----
// ----  Render objects with basic diffuse lighting        ----
// ----  and color coded debug index                       ----
// ------------------------------------------------------------

#ifdef DEBUG_LIGHT_AND_SHADOW

uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldIT;
uniform highp mat4 World;


#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		vec4	vPosWS;		// World Space position

void main()
{
	vPosWS = World * Vertex;
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec3 sunLightDir; // WS normalized

uniform highp mat4 ViewToLightMatrix;
uniform highp mat4 LightProjMatrix;

uniform highp sampler2D ShadowMap;

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space normal
varying highp		vec4    vPosWS;		// World Space position

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// Return 1.0 if visible, 0.0 if occluded
mediump float computeVisibility()
{
	highp float bias = 0.001;
	
	// Compute vertex position in LS
	highp vec4 posLS = LightProjMatrix * (ViewToLightMatrix * vPosWS);
	
	// No need for perspective division for a directional light
	//posLS /= posLS.w;
	
	posLS.xyz = posLS.xyz * 0.5 + 0.5;
	posLS.y = 1.0 - posLS.y;
	posLS.z -= bias;
	
	// Fetch shadow map
	highp float currentDepth = texture2D(ShadowMap, posLS.xy).r;
	
	return float(currentDepth > posLS.z);
}

void main()
{
	mediump float lightIntensity = 0.0;
	
	lightIntensity = computeLighting();
	
	highp float visibility = computeVisibility();
	
	// A bit of aesthetic ;)
	const lowp vec3 diffuseColor = vec3(0.94,0.84,0.19);
	const lowp vec4 shadowColor = vec4(6.0 / 255.0,  7.0 / 255.0, 60.0 / 255.0, 0.8);
	
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	lowp vec3 finalColor = mix(diffuseColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// DEBUG_LIGHT_AND_SHADOW

// ------------------------------------------------------------
// ----               LIGHT_AND_SHADOW                     ----
// ----  Render objects with basic diffuse lighting        ----
// ----                                                    ----
// ------------------------------------------------------------

#ifdef LIGHT_AND_SHADOW

uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldIT;
uniform highp mat4 World;


#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		vec4	vPosWS;		// World Space position

void main()
{
	vPosWS = World * Vertex;
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec3 sunLightDir; // WS normalized

uniform highp mat4 ViewToLightMatrix;
uniform highp mat4 LightProjMatrix;

uniform highp sampler2D ShadowMap;

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space normal
varying highp		vec4    vPosWS;		// World Space position

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// Return 1.0 if visible, 0.0 if occluded
mediump float computeVisibility()
{
	highp float bias = 0.001;
	
	// Compute vertex position in LS
	highp vec4 posLS = LightProjMatrix * (ViewToLightMatrix * vPosWS);
	
	posLS.xyz = posLS.xyz * 0.5 + 0.5;
	posLS.y = 1.0 - posLS.y;
	posLS.z -= bias;
	
	// Fetch proper cascade
	highp float currentDepth = texture2D(ShadowMap, posLS.xy).r;
	
	return float(currentDepth > posLS.z);
}

void main()
{
	mediump float lightIntensity = 0.0;
	
	lightIntensity = computeLighting();
	
	highp float visibility = computeVisibility();
	
	const lowp vec3 diffuseColor = vec3(0.8,0.8,0.8);
		
	const lowp vec4 shadowColor = vec4(0.0,0.0,0.0,0.8);
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	lowp vec3 finalColor = mix(diffuseColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// LIGHT_AND_SHADOW

// ------------------------------------------------------------
// ----              DEBUG_LIGHT_AND_SHADOW_HW                ----
// ----  Render objects with basic diffuse lighting        ----
// ----  and color coded debug index                       ----
// ------------------------------------------------------------

#ifdef DEBUG_LIGHT_AND_SHADOW_HW

#if GL_ES
#extension GL_EXT_shadow_samplers : require
#endif

uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldIT;
uniform highp mat4 World;


#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		vec4	vPosWS;		// World Space position

void main()
{
	vPosWS = World * Vertex;
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec3 sunLightDir; // WS normalized

uniform highp mat4 ViewToLightMatrix;
uniform highp mat4 LightProjMatrix;

uniform highp sampler2DShadow ShadowMap;

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space normal
varying highp		vec4    vPosWS;		// World Space position

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// Return 1.0 if visible, 0.0 if occluded
mediump float computeVisibility()
{
	highp float bias = 0.001;
	
	// Compute vertex position in LS
	highp vec4 posLS = LightProjMatrix * (ViewToLightMatrix * vPosWS);
	
	// No need for perspective division for a directional light
	//posLS /= posLS.w;
	
	posLS.xyz = posLS.xyz * 0.5 + 0.5;
	posLS.y = 1.0 - posLS.y;
	posLS.z -= bias;
	
#if GL_ES
	return shadow2DEXT(ShadowMap, posLS.xyz);
#else
	return shadow2D(ShadowMap, posLS.xyz).r;
#endif

}

void main()
{
	mediump float lightIntensity = 0.0;
	
	lightIntensity = computeLighting();
	
	highp float visibility = computeVisibility();
	
	// A bit of aesthetic ;)
	const lowp vec3 diffuseColor = vec3(0.94,0.84,0.19);
	const lowp vec4 shadowColor = vec4(6.0 / 255.0,  7.0 / 255.0, 60.0 / 255.0, 0.8);
	
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	lowp vec3 finalColor = mix(diffuseColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// DEBUG_LIGHT_AND_SHADOW_HW

// ------------------------------------------------------------
// ----               LIGHT_AND_SHADOW_HW                    ----
// ----  Render objects with basic diffuse lighting        ----
// ----                                                    ----
// ------------------------------------------------------------

#ifdef LIGHT_AND_SHADOW_HW

#if GL_ES
#extension GL_EXT_shadow_samplers : require
#endif

uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldIT;
uniform highp mat4 World;


#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		vec4	vPosWS;		// World Space position

void main()
{
	vPosWS = World * Vertex;
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec3 sunLightDir; // WS normalized

uniform highp mat4 ViewToLightMatrix;
uniform highp mat4 LightProjMatrix;

uniform highp sampler2DShadow ShadowMap;

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space normal
varying highp		vec4    vPosWS;		// World Space position

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// Return 1.0 if visible, 0.0 if occluded
mediump float computeVisibility()
{
	highp float bias = 0.001;
	
	// Compute vertex position in LS
	highp vec4 posLS = LightProjMatrix * (ViewToLightMatrix * vPosWS);
	
	posLS.xyz = posLS.xyz * 0.5 + 0.5;
	posLS.y = 1.0 - posLS.y;
	posLS.z -= bias;
	
#if GL_ES
	return shadow2DEXT(ShadowMap, posLS.xyz);
#else
	return shadow2D(ShadowMap, posLS.xyz).r;
#endif
}

void main()
{
	mediump float lightIntensity = 0.0;
	
	lightIntensity = computeLighting();
	
	highp float visibility = computeVisibility();
	
	const lowp vec3 diffuseColor = vec3(0.8,0.8,0.8);
		
	const lowp vec4 shadowColor = vec4(0.0,0.0,0.0,0.8);
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	lowp vec3 finalColor = mix(diffuseColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// LIGHT_AND_SHADOW_HW

// --------------------------------------------
// ----           DEBUG_RENDER_LISPSM      ----
// ----  Transform geometry using LiSP     ----
// --------------------------------------------

#ifdef DEBUG_RENDER_LISPSM

uniform highp mat4 WorldViewProjectionMatrix;

#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

void main()
{
	// Snap objects that are in light frustrum but not in lisp
	highp vec4 transformedVertex = WorldViewProjectionMatrix * Vertex;
	transformedVertex.z = max(-1.0, transformedVertex.z / transformedVertex.w) * transformedVertex.w;
	
	gl_Position = transformedVertex;
}
#endif

#ifdef FRAGMENT_SHADER
void main()
{
	// Write any color coded debug value in color RT
	const lowp vec4 green = vec4(0.0, 1.0, 0.0, 1.0);
	gl_FragColor = green;
}
#endif
// FRAGMENT_SHADER

#endif 
// DEBUG_RENDER_SM

// --------------------------------------------
// ----           RENDER_LISPSM            ----
// ----  Transform geometry using LiSP     ----
// --------------------------------------------

#ifdef RENDER_LISPSM

uniform highp mat4 WorldViewProjectionMatrix;

#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

void main()
{
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

#ifdef FRAGMENT_SHADER
void main()
{
	// Write any color coded debug value in color RT
	gl_FragColor = vec4(0.0, 1.0, 0.0, 1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// RENDER_SM

// ------------------------------------------------------------
// ----              DEBUG_LIGHT_AND_LISPSHADOW            ----
// ----  Render objects with basic diffuse lighting        ----
// ----  and color coded debug index                       ----
// ------------------------------------------------------------

// Currently debug and release shader share same code
#ifdef LIGHT_AND_LISPSHADOW
#define DEBUG_LIGHT_AND_LISPSHADOW
#endif

#ifdef DEBUG_LIGHT_AND_LISPSHADOW

uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldIT;
uniform highp mat4 World;


#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		vec4	vPosWS;		// World Space position

void main()
{
	vPosWS = World * Vertex;
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec3 sunLightDir; // WS normalized
uniform highp mat4 LiSPMatrix;

uniform highp sampler2D ShadowMap;

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space normal
varying highp		vec4    vPosWS;		// World Space position

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// Return 1.0 if visible, 0.0 if occluded
mediump float computeVisibility()
{
	// Should be slope-based
	highp float bias = 0.001;
	
	// Compute vertex position in LS
	highp vec4 posLS =  LiSPMatrix * vPosWS;
	posLS /= posLS.w;
	
	posLS.xyz = posLS.xyz * 0.5 + 0.5;
	posLS.y = 1.0 - posLS.y;
	posLS.z -= bias;
	
	// Fetch shadow map
	highp float currentDepth = texture2D(ShadowMap, posLS.xy).r;
	
	return float(currentDepth > posLS.z);
}

void main()
{
	mediump float lightIntensity = 0.0;
	
	lightIntensity = computeLighting();
	
	highp float visibility = computeVisibility();
	
	// A bit of aesthetic ;)
	const lowp vec3 diffuseColor = vec3(0.84,0.94,0.19);
	const lowp vec4 shadowColor = vec4(60.0 / 255.0,  7.0 / 255.0, 6.0 / 255.0, 0.8);
	
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	lowp vec3 finalColor = mix(diffuseColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// DEBUG_LIGHT_AND_LISPSHADOW

// ------------------------------------------------------------
// ----              DEBUG_LIGHT_AND_LISPSHADOW_HW           ----
// ----  Render objects with basic diffuse lighting        ----
// ----  and color coded debug index                       ----
// ------------------------------------------------------------

#ifdef LIGHT_AND_LISPSHADOW_HW
#define DEBUG_LIGHT_AND_LISPSHADOW_HW
#endif

#ifdef DEBUG_LIGHT_AND_LISPSHADOW_HW

#if GL_ES
#extension GL_EXT_shadow_samplers : require
#endif

uniform highp mat4 WorldViewProjectionMatrix;
uniform highp mat4 WorldIT;
uniform highp mat4 World;

#ifdef VERTEX_SHADER
attribute highp 	vec4 Vertex
#ifdef DCC_MAX
: POSITION
#endif
;

attribute highp 	vec3 Normal
#ifdef DCC_MAX
: NORMAL
#endif
;

varying mediump		vec3	vNormalWS;	// World Space
varying highp		vec4	vPosWS;		// World Space position

void main()
{
	vPosWS = World * Vertex;
	vNormalWS = (WorldIT * vec4(Normal, 1.0)).xyz;
	gl_Position = WorldViewProjectionMatrix * Vertex;
}
#endif 
// VERTEX_SHADER

uniform mediump vec3 sunLightDir; // WS normalized
uniform highp mat4 LiSPMatrix;

uniform highp sampler2DShadow ShadowMap;

#ifdef FRAGMENT_SHADER

varying mediump		vec3	vNormalWS;	// World space normal
varying highp		vec4    vPosWS;		// World Space position

mediump float computeLighting() 
{
	mediump vec3 normal = normalize(vNormalWS);
	return max(0.0, dot(normal,sunLightDir));
}

// Return 1.0 if visible, 0.0 if occluded
mediump float computeVisibility()
{
	highp float bias = 0.001;
	
	// Compute vertex position in LS
	highp vec4 posLS =  LiSPMatrix * vPosWS;
	posLS /= posLS.w;
	
	posLS.xyz = posLS.xyz * 0.5 + 0.5;
	posLS.y = 1.0 - posLS.y;
	posLS.z -= bias;
	
	//if ((posLS.x > 0.0) && (posLS.x < 1.0) && (posLS.y > 0.0) && (posLS.y < 1.0))
	//{
	
#if GL_ES
	return shadow2DEXT(ShadowMap, posLS.xyz);
#else
	return shadow2D(ShadowMap, posLS.xyz).r;
#endif
	//}
	//else
	//	return 1.0;

}

void main()
{
	mediump float lightIntensity = 0.0;
	
	lightIntensity = computeLighting();
	
	highp float visibility = computeVisibility();
	
	// A bit of aesthetic ;)
	const lowp vec3 diffuseColor = vec3(0.84,0.94,0.19);
	const lowp vec4 shadowColor = vec4(60.0 / 255.0,  7.0 / 255.0, 6.0 / 255.0, 0.8);
	
	mediump float shadowIntensity = min(1.0 - visibility, shadowColor.a);
	lowp vec3 finalColor = mix(diffuseColor * lightIntensity, shadowColor.rgb, shadowIntensity);
	
	gl_FragColor = vec4(finalColor,1.0);
}
#endif
// FRAGMENT_SHADER

#endif 
// DEBUG_LIGHT_AND_LISPSHADOW_HW
