uniform lowp mat4 ColorMatrix; // Android

#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

#if defined AT
	uniform lowp float alpharef;
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);
varying mediump vec2 vTexCoord1 DCC(: TEXCOORD1);

varying lowp vec4 vColor DCC(: COLOR0);

uniform lowp sampler2D DiffuseSampler;
uniform lowp sampler2D DiffuseSampler_alpha; // Android

#ifdef AL
	 uniform lowp float Object_Alpha;
#endif

uniform lowp sampler2D LightMapSampler;
uniform lowp sampler2D LightMapSampler_alpha;

uniform lowp vec3 lmpercentage0;
uniform lowp vec3 lmpercentage1;
 
#ifdef FG
	varying lowp float vFogFactor;
	uniform lowp vec4 fogcolor;   
#endif

uniform lowp vec4 sunFactor;

#define TONED
#ifdef TONED
	uniform lowp vec3 ToneGain;
	uniform lowp vec3 ToneOffset;
#endif

void main()
{
	lowp vec4 Diffuse = texture2D(DiffuseSampler, vTexCoord0);	
 
#ifdef AT
	if (Diffuse.w < alpharef) discard; 
#endif

	//lowp vec4 lightColor = vColor;
	
	//removed dynamic lighting...
	lowp vec4 lightColor = vec4(1.0, 1.0, 1.0, 1.0);
 
 	lowp vec4 color = lightColor * Diffuse;

	lowp vec3 lmcolor = ( texture2D(LightMapSampler, vTexCoord1).rgb * lmpercentage0
	#ifdef DCC_MAX
						//want lmpercentage1 bound, but not LightMapSampler_alpha (maxGlitch issue, it's supposed to be an automatic)
						+ texture2D(LightMapSampler, vTexCoord1).rgb * lmpercentage1 
	#else
						+ texture2D(LightMapSampler_alpha, vTexCoord1).rgb * lmpercentage1
	#endif
						);

	color.rgb *= lmcolor;

#ifdef AL
	color.a *= Object_Alpha;
#endif

	color.rgb += sunFactor.rgb;

#ifdef FG
	color.xyz = mix(color.xyz, fogcolor.xyz, vFogFactor);
#endif
 
#ifdef TONED	
	color.rgb = color.rgb * ToneGain + ToneOffset;
#endif
	color.a = texture2D(DiffuseSampler_alpha, vTexCoord0).r; // Android
	gl_FragColor = clamp(color, 0.0, 1.0);
} // main end
