#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

attribute highp vec4 Vertex DCC(: POSITION);
attribute highp vec3 Normal DCC(: NORMAL);

attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);
attribute mediump vec2 texcoord1 DCC(: TEXCOORD1);

#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET0)
	uniform mediump vec4 texcoord0_scaleoffset;
#endif
#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET1)
	uniform mediump vec4 texcoord1_scaleoffset;
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);
varying mediump vec2 vTexCoord1 DCC(: TEXCOORD1);

varying lowp vec4 vColor DCC(: COLOR0);

uniform highp vec3 Light0DirectionOS;
uniform lowp vec4 Light0AmbientColor;
uniform lowp vec4 Light0DiffuseColor;

uniform highp vec3 Light1DirectionOS;
uniform lowp vec4 Light1AmbientColor;
uniform lowp vec4 Light1DiffuseColor;

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

uniform highp mat4 matWorldViewProjection;
uniform highp mat4 matWorldIT;

void main()
{	
	// Vertex Position output
	gl_Position = matWorldViewProjection * Vertex;

    highp vec3 transformedNormal = (matWorldIT * vec4(Normal, 0.0)).xyz;
 
/////////////////////////
//Lighting Output	   //						  	
/////////////////////////
	lowp float nDotVP = max(0.0, dot(Normal, Light1DirectionOS));
    lowp vec4 Diffuse = Light1DiffuseColor * nDotVP;

	//calculate first light last, as we need to keep nDotVP for the spec calculation
	nDotVP = max(0.0, dot(Normal, Light0DirectionOS));
    Diffuse += Light0DiffuseColor * nDotVP;

    // Specular lighting will be calculated in the fragment
    lowp vec4 color = vec4(Light0AmbientColor.xyz + Diffuse.xyz, 1.0);
    
	//removed dynamic lighting...
	//vColor = color;

/////////////////////////
//UV coordinates output//						  	
/////////////////////////
	vTexCoord0 = texcoord0;	// Diffuse/Specular
#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET0)
	vTexCoord0 = vTexCoord0 * texcoord0_scaleoffset.xy + texcoord0_scaleoffset.zw;
#endif

	vTexCoord1 = texcoord1;	// LightMap
#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET1)
	vTexCoord1 = vTexCoord1 * texcoord1_scaleoffset.xy + texcoord1_scaleoffset.zw;
#endif

/////////////////////////
//Fog factor output	   //
/////////////////////////
#ifdef FG
	highp float fogDistance = dot(matWorldViewT[2], Vertex);//length(fogVector);
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
} // main end
