uniform lowp mat4 ColorMatrix; // Android

#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

#if defined AT
	uniform lowp float alpharef;
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

varying lowp vec4 vColor DCC(: COLOR0);

uniform lowp sampler2D DiffuseSampler;
uniform lowp sampler2D DiffuseSampler_alpha; // Android

#if defined (AL) || defined (AB)
	uniform lowp float Object_Alpha;
#endif

uniform lowp vec4 sunFactor;

#ifdef VERTEX_ALPHA
	varying lowp float vAlpha;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform lowp vec4 fogcolor;   
#endif

#ifdef COLOR_VAR
	uniform lowp sampler2D ColorVarMask;
	uniform lowp vec3 ColorVar1;
	uniform lowp vec3 ColorVar2;
	uniform lowp vec3 ColorVar3;
	uniform lowp vec3 ColorVar4;
#endif

#define TONED
#ifdef TONED
	uniform lowp vec3 ToneGain;
	uniform lowp vec3 ToneOffset;
#endif

void main()
{
	//lowp vec4 Diffuse = texture2D(DiffuseSampler, vTexCoord0); // Android
	lowp vec4 Diffuse = vec4( texture2D(DiffuseSampler, vTexCoord0).rgb, texture2D(DiffuseSampler_alpha, vTexCoord0).g ) ; // Android

#ifdef AT
	if (Diffuse.w < alpharef) discard;	
#endif

#ifdef COLOR_VAR
	lowp vec4 ColorMask = texture2D(ColorVarMask, vTexCoord0);
	
	lowp vec3 diffuseColor = Diffuse.rgb;
	diffuseColor += ColorVar1 * ColorMask.x;
	diffuseColor += ColorVar2 * ColorMask.y;
	diffuseColor += ColorVar3 * ColorMask.z;
	diffuseColor += ColorVar4 * ColorMask.w;
	
	Diffuse.rgb = clamp(diffuseColor, 0.0, 1.0);
#endif

	lowp vec4 color = vColor * Diffuse;

#ifdef AL
	color.a *= Object_Alpha;
#endif

#ifdef AB
	color.rgb *= Object_Alpha;
#endif

#ifdef VERTEX_ALPHA
	color.a *= vAlpha;
#else
	color.rgb += sunFactor.rgb;
#endif

#ifdef FG
#ifdef ADDITIVE_BLENDING
	color.xyz = mix(color.xyz, vec3(0.0, 0.0, 0.0), vFogFactor);
#else // ADDITIVE_BLENDING
	color.xyz = mix(color.xyz, fogcolor.xyz, vFogFactor);
#endif // ADDITIVE_BLENDING
#endif // FG

#ifdef TONED	
	color.rgb = color.rgb * ToneGain + ToneOffset;
#endif

	gl_FragColor = clamp(color, 0.0, 1.0);
} // main end
