#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

attribute highp vec4 Vertex DCC(: POSITION);

attribute highp vec3 Normal DCC(: NORMAL);

attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);
#ifdef UV_SCALE_OFFSET
	uniform mediump vec4 texcoord0_scaleoffset;
#endif

#ifdef VERTEX_ALPHA
	#ifdef DCC_MAX
		attribute lowp    vec3 Color DCC(: TEXCOORD1);
		attribute lowp    float Alpha DCC(: TEXCOORD2);
	#else
		attribute lowp    vec4 Color;
	#endif
	varying lowp float vAlpha;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

varying lowp vec4 vColor DCC(: COLOR0);

uniform highp vec3 Light0DirectionOS;
uniform lowp vec4 Light0AmbientColor;
uniform lowp vec4 Light0DiffuseColor;

uniform highp vec3 Light1DirectionOS;
uniform lowp vec4 Light1DiffuseColor;

uniform lowp float SunOcclusionFactor;

#ifdef UV
	uniform mediump float offsetU;
	uniform mediump float offsetV;
#endif

uniform highp mat4 matWorldViewProjection;

#ifdef TREE
	uniform highp vec2 FadeParams;
#endif

void main()
{	
	// Vertex Position output
	gl_Position = matWorldViewProjection * Vertex;

#ifndef NO_DIFFUSE
    lowp float nDotVP = max(0.0, dot(Normal, Light0DirectionOS)); 
    lowp vec3 Diffuse = Light0DiffuseColor.rgb * nDotVP;
    nDotVP = max(0.0, dot(Normal, Light1DirectionOS)); 
    Diffuse +=  Light1DiffuseColor.rgb * nDotVP;
#else
	lowp vec3 Diffuse = vec3(0.0, 0.0, 0.0);
#endif

	lowp vec3 Ambient = Light0AmbientColor.rgb;
	
#ifdef VERTEX_ALPHA
	#ifdef DCC_MAX
		vAlpha = Alpha;
	#else
		vAlpha = Color.a;
	#endif
    lowp vec4 color = vec4(Ambient.xyz + Diffuse.xyz, 1.0);
#else
    lowp vec4 color = vec4(Ambient.xyz + Diffuse.xyz * SunOcclusionFactor, 1.0);
#endif
	vColor = color;
	
	#ifdef DCC_MAX
		if (vColor.rgb == vec3(0.0))
   	 		vColor = vec4(1.0);
    #endif

	vTexCoord0 = texcoord0;	// Diffuse/Specular
	
#ifdef UV
	//UV Animations
	vTexCoord0.x += offsetU;
	vTexCoord0.y += offsetV;
#endif

#ifdef UV_SCALE_OFFSET
	vTexCoord0 = vTexCoord0 * texcoord0_scaleoffset.xy + texcoord0_scaleoffset.zw;
#endif

/////////////////////////
//Fog factor output	   //
/////////////////////////
#ifdef FG
	highp float fogDistance = dot(matWorldViewT[2], Vertex);
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
} // main end
