#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

//Fel| Fix for PowerSGX544MP
#ifndef USE_DIFFUSE_SAMPLER_FOR_SPECULAR
#define USE_DIFFUSE_SAMPLER_FOR_SPECULAR
#endif

#ifdef USE_SPECULAR_SAMPLER
#undef USE_DIFFUSE_SAMPLER_FOR_SPECULAR
#endif

#if defined(REQUEST_USE_LIGHTMAP) && !defined(BYPASS_USE_LIGHTMAP)
#define USE_LIGHTMAP
#endif

//Fel| Fix for PowerSGX544MP
#ifndef REQUEST_USE_LIGHTMAP
#define REQUEST_USE_LIGHTMAP
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);
varying mediump vec2 vTexCoord1 DCC(: TEXCOORD1);

#ifdef USE_TRANSITION_TEXTURE
	uniform lowp sampler2D TransitionSampler;
#endif // USE_TRANSITION_TEXTURE

varying lowp vec4 vColor DCC(: COLOR0);

uniform lowp sampler2D DiffuseSampler;

#if defined(USE_VERTEX_PAINT_DIFFUSE_MIX)
uniform lowp sampler2D DiffuseSamplerG;
uniform lowp sampler2D DiffuseSamplerB;
#endif

#if defined(SPECULAR) && defined(DISABLE_SPECULAR)
	#undef SPECULAR
	#define USE_SPEC_MAP
#endif

#ifdef SPECULAR
	uniform lowp sampler2D SpecularSampler;
	uniform lowp vec4 Light0Specularcolor;	
	uniform highp float Glossiness;
	uniform highp float SpecFactor;
	varying highp vec3 vSpecN; 	
	varying highp vec3 vVertexToEye;
	varying highp vec3 vVertexToLight;
	
#if defined(USE_VERTEX_PAINT_SPECULAR_MIX)
	uniform lowp sampler2D SpecularSamplerG;
	uniform lowp sampler2D SpecularSamplerB;
#endif
	
#endif

#ifdef USE_LIGHTMAP
	uniform lowp sampler2D LightMapSampler;
	uniform lowp sampler2D LightMapSampler_alpha;
#endif

uniform lowp float lmboost;

uniform lowp vec3 lmpercentage0;
uniform lowp vec3 lmpercentage1;
 
#ifdef ROAD_RF
	uniform lowp float RoadReflectionAlphaMin;
	uniform lowp float RoadReflectionAlphaMax;	 
	#ifdef USE_SPEC_MAP
		uniform lowp sampler2D SpecularSampler;
	#endif
#endif 
 
#ifdef FG
	varying lowp float vFogFactor;
	uniform lowp vec4 fogcolor;   
#endif

uniform lowp vec4 sunFactor;

//Fel| Fix for PowerSGX544MP
#ifndef TONED
#define TONED
#endif


#ifdef TONED
	uniform lowp vec3 ToneGain;
	uniform lowp vec3 ToneOffset;
#endif
 
void main()
{
#ifdef USE_TRANSITION_TEXTURE
	lowp vec4 Diffuse1 = texture2D(DiffuseSampler, vTexCoord0);
	lowp vec4 Diffuse2 = texture2D(TransitionSampler, vTexCoord0);
	
	lowp vec4 Diffuse = mix(Diffuse1, Diffuse2, vColor.g);
#else
#ifdef USE_VERTEX_PAINT_DIFFUSE_MIX
	lowp float DiffuseTexR = texture2D(DiffuseSampler, vTexCoord0).r;
	lowp float DiffuseTexG = texture2D(DiffuseSamplerG, vTexCoord0).r;
	lowp float DiffuseTexB = texture2D(DiffuseSamplerB, vTexCoord0).r;
	lowp vec3 DiffuseTex = vec3(DiffuseTexR, DiffuseTexG, DiffuseTexB);
	lowp float DiffuseValue = dot(DiffuseTex, vColor.rgb);	
	lowp vec4 Diffuse = vec4(DiffuseValue, DiffuseValue, DiffuseValue, 1.0);
#else
	lowp vec4 Diffuse = texture2D(DiffuseSampler, vTexCoord0);
#endif	
#endif

 	lowp vec4 color = Diffuse;

#ifdef USE_LIGHTMAP
	lowp vec3 lmcolor = ( texture2D(LightMapSampler, vTexCoord1).rgb * lmpercentage0
	#ifdef DCC_MAX
						//want lmpercentage1 bound, but not LightMapSampler_alpha (maxGlitch issue, it's supposed to be an automatic)
						+ texture2D(LightMapSampler, vTexCoord1).rgb * lmpercentage1 
	#else
						+ texture2D(LightMapSampler_alpha, vTexCoord1).rgb * lmpercentage1
	#endif
						);

	color.rgb *= lmcolor;
		
#endif

#ifdef SPECULAR
#if defined(USE_DIFFUSE_SAMPLER_FOR_SPECULAR) && !defined(DCC_MAX)
	lowp float Specular = Diffuse.r;
#else
	#ifdef USE_VERTEX_PAINT_SPECULAR_MIX
		lowp float SpecularTexR = texture2D(SpecularSampler, vTexCoord0).r;
		lowp float SpecularTexG = texture2D(SpecularSamplerG, vTexCoord0).r;
		lowp float SpecularTexB = texture2D(SpecularSamplerB, vTexCoord0).r;
		lowp vec3 SpecularTex = vec3(SpecularTexR, SpecularTexG, SpecularTexB);	
	
		lowp float Specular = dot(SpecularTex.rgb, vColor.rgb);
	#else
		lowp float Specular = texture2D(SpecularSampler, vTexCoord0).r;
	#endif
#endif
	// Get Specular Color
	lowp vec3 SpecLight = Light0Specularcolor.xyz;
	
	// Normalize light and eye vectors
	highp vec3 L = normalize(vVertexToLight.xyz);
	highp vec3 V = normalize(vVertexToEye.xyz);
	
	// Compute Half Vector
    highp vec3 H = normalize(L + V);

	// Get Specular component
	highp float NDotH = pow(max(0.0, dot(H, vSpecN.xyz)), Glossiness);	
	highp vec3 specularColor = Specular * SpecLight * SpecFactor;	 	
	specularColor *= NDotH;

#ifdef USE_LIGHTMAP
	lowp float averageLM = dot(lmcolor.xyz, vec3(0.33, 0.33, 0.33));
	specularColor *= min(0.25, averageLM * averageLM);
#endif
	
	color.rgb += specularColor;
#endif

	color.rgb += sunFactor.rgb;

#ifdef ROAD_RF
	lowp float finalReflAlpha = vColor.a;
	#ifdef USE_SPEC_MAP
		lowp float Specular = texture2D(SpecularSampler, vTexCoord0).x;
		finalReflAlpha += mix(RoadReflectionAlphaMin, RoadReflectionAlphaMax, Specular) * (1.0 - vColor.a);
	#else
		finalReflAlpha += RoadReflectionAlphaMin * (1.0 - vColor.a);
	#endif
	
	#ifdef DCC_MAX
		finalReflAlpha += RoadReflectionAlphaMax * 0.0001;
	#endif	
	
	color.a = finalReflAlpha;

#ifdef FG
	color.a = mix(color.a, 1.0, vFogFactor);
#endif
	
#endif

#ifdef FG
	color.xyz = mix(color.xyz, fogcolor.xyz, vFogFactor);
#endif

#ifdef TONED	
	color.rgb = color.rgb * ToneGain + ToneOffset;
#endif
	gl_FragColor = color;		
} // main end
