#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

#if defined(SPECULAR) && defined(DISABLE_SPECULAR)
	#undef SPECULAR
#endif

attribute highp vec4 Vertex DCC(: POSITION);
attribute highp vec3 Normal DCC(: NORMAL);

#ifndef DCC_MAX
	attribute lowp vec4 Color DCC(: COLOR);
#endif

attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);
attribute mediump vec2 texcoord1 DCC(: TEXCOORD1);

#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET0)
	uniform mediump vec4 texcoord0_scaleoffset;
#endif
#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET1)
	uniform mediump vec4 texcoord1_scaleoffset;
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);
varying mediump vec2 vTexCoord1 DCC(: TEXCOORD1);

#ifdef SPECULAR
	varying highp float vAttNdotHV;	
	varying highp vec3 vSpecN;	
	
	varying highp vec3 vVertexToEye;
	varying highp vec3 vVertexToLight;
	
#endif

uniform highp vec3 Light0PositionOS;
uniform highp vec3 Light0Direction; 
uniform highp vec3 Light0DirectionOS;
uniform lowp vec4 Light0AmbientColor;
uniform lowp vec4 Light0DiffuseColor;

uniform highp vec3 Light1Direction; 
uniform highp vec3 Light1DirectionOS; 
uniform lowp vec4 Light1AmbientColor;
uniform lowp vec4 Light1DiffuseColor;

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend; 
	uniform highp mat4 matWorldViewT; 
#endif

uniform highp mat4 matWorldViewProjection;  
uniform highp mat4 matWorldIT;
uniform highp mat4 matWorld;

uniform highp mat4 matView;
uniform highp mat4 matViewIT;

uniform highp mat4 matWorldView;
uniform highp mat4 matWorldViewIT;

varying lowp vec4 vColor DCC(: COLOR0);

void main()
{	
	// Vertex Position output
	gl_Position = matWorldViewProjection * Vertex;    
 
/////////////////////////
//Lighting Output	   //						  	
/////////////////////////
	highp float nDotVP = max(0.0, dot(Normal, Light1DirectionOS));
    highp vec4 Diffuse = Light1DiffuseColor * nDotVP;

	//calculate first light last, as we need to keep nDotVP for the spec calculation
	nDotVP = max(0.0, dot(Normal, Light0DirectionOS));
    Diffuse += Light0DiffuseColor * nDotVP;

    // Specular lighting will be calculated in the fragment
    highp vec4 color = vec4(Light0AmbientColor.xyz + Diffuse.xyz, 1.0);
    
    //removed dynamic lighting...
	//vColor = color;

//only calculate specular using light 0.
#ifdef SPECULAR
	// Compute Specular data in View Space	

	
	vVertexToEye = -((matWorldView * Vertex).xyz * 0.01);
	vVertexToLight = normalize(((matWorldViewIT) * vec4(Light0PositionOS, 1.0)).xyz);

	vSpecN = normalize(((matWorldViewIT) * vec4(Normal, 0.0)).xyz);		

#endif
/////////////////////////
//UV coordinates output//						  	
/////////////////////////
	vTexCoord0 = texcoord0;	// Diffuse/Specular
#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET0)
	vTexCoord0 = vTexCoord0 * texcoord0_scaleoffset.xy + texcoord0_scaleoffset.zw;
#endif

	vTexCoord1 = texcoord1;	// LightMap
#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET1)
	vTexCoord1 = vTexCoord1 * texcoord1_scaleoffset.xy + texcoord1_scaleoffset.zw;
#endif

#ifndef DCC_MAX
	vColor = Color;		
	
#endif

/////////////////////////
//Fog factor output	   //
/////////////////////////
#ifdef FG
	highp float fogDistance = dot(matWorldViewT[2], Vertex);//length(fogVector);    //////////////////////////////////////////////////////////////////////////////////////////
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
} // main end
