uniform lowp mat4 ColorMatrix; // Android

#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG
//#define LM

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

varying lowp vec4 vColor DCC(: COLOR0);

uniform lowp sampler2D DiffuseSampler;
uniform lowp sampler2D DiffuseSampler_alpha; // Android

#ifdef LM
uniform lowp sampler2D LightMapSampler;
uniform lowp sampler2D LightMapSampler_alpha;

uniform lowp vec3 lmpercentage0;
uniform lowp vec3 lmpercentage1;

varying mediump vec2 vTexCoord1 DCC(: TEXCOORD1);
#endif // LM

uniform lowp float Object_Alpha;

uniform lowp vec4 sunFactor;

#ifdef FG
	varying lowp float vFogFactor;
	uniform lowp vec4 fogcolor;   
#endif

#define TONED
#ifdef TONED
	uniform lowp vec3 ToneGain;
	uniform lowp vec3 ToneOffset;
#endif

void main()
{
	//lowp vec4 Diffuse = texture2D(DiffuseSampler, vTexCoord0); // Android
	lowp vec4 Diffuse = vec4( texture2D(DiffuseSampler, vTexCoord0).rgb, texture2D(DiffuseSampler_alpha, vTexCoord0).g ) ; // Android

	lowp vec4 color = vColor * Diffuse;

#ifdef LM
	lowp vec3 lmcolor = ( texture2D(LightMapSampler, vTexCoord1).rgb * lmpercentage0
	#ifdef DCC_MAX
						//want lmpercentage1 bound, but not LightMapSampler_alpha (maxGlitch issue, it's supposed to be an automatic)
						+ texture2D(LightMapSampler, vTexCoord1).rgb * lmpercentage1 
	#else
						+ texture2D(LightMapSampler_alpha, vTexCoord1).rgb * lmpercentage1
	#endif
						);
						
	color.rgb *= lmcolor;
#endif // LM

	color.a *= Object_Alpha;
	
	color.rgb += sunFactor.rgb;
	
#ifdef FG
	color.xyz = mix(color.xyz, fogcolor.xyz, vFogFactor);
#endif

#ifdef TONED	
	color.rgb = color.rgb * ToneGain + ToneOffset;
#endif

	gl_FragColor = clamp(color, 0.0, 1.0);
} // main end
