#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG
//#define LM

attribute highp vec4 Vertex DCC(: POSITION);

attribute highp vec3 Normal DCC(: NORMAL);

#ifdef DCC_MAX
	attribute lowp    vec3 Color DCC(: TEXCOORD2);
#else
	attribute lowp    vec4 Color;
#endif


attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);

#ifdef LM
attribute mediump vec2 texcoord1 DCC(: TEXCOORD1);
#endif // LM

#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET0)
uniform mediump vec4 texcoord0_scaleoffset;
#endif

#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET1)
	uniform mediump vec4 texcoord1_scaleoffset;
#endif

uniform highp float time;

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

#ifdef LM
varying mediump vec2 vTexCoord1 DCC(: TEXCOORD1);
#endif // LM

varying lowp vec4 vColor DCC(: COLOR0);

uniform highp vec3 Light0DirectionOS;
uniform lowp vec4 Light0AmbientColor;
uniform lowp vec4 Light0DiffuseColor;

uniform highp vec3 Light1DirectionOS;
uniform lowp vec4 Light1DiffuseColor;

uniform lowp float SunOcclusionFactor;

uniform highp mat4 matWorldViewProjection;
uniform highp mat4 world;

uniform highp vec2 FadeParams;

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

#define ANIMATE_VERTEX

#ifdef ANIMATE_VERTEX
	uniform mediump vec4 windParams; //(wind Strength, Branch Amplitude, Detail Amplitude, Speed)
	uniform mediump vec4 windParams2; //wind direction (xyz), bendFactor
	
	const vec2 detailFrequencies = vec2(1.975, 0.793);
	const vec2 branchFrequencies = vec2(0.375, 0.193)*1.5;
	const vec2 trunkFrequencies = vec2(0.123, 0.0877)*3.0;
#endif
void main()
{	
#ifdef ANIMATE_VERTEX
	float windStrength = windParams.x;
	float speedModifier = windParams.w;

	float globalBranchAmplitude = windParams.y;
	float globalDetailAmplitude = windParams.z;
	float vertexDetailAmplitude = Color.r;
	float vertexBranchAmplitude = Color.b;
	float detailAmplitude = globalDetailAmplitude * vertexDetailAmplitude * vertexBranchAmplitude;
	float branchAmplitude = globalBranchAmplitude * vertexBranchAmplitude;
	
	vec3 position = Vertex.xyz;
	vec4 wPosition = world * Vertex;
	
	// Phases
	float vertexPhase = dot(position.xyz*0.01, vec3(Color.g)); 	
	float mainPhase = time * speedModifier;
	vec2 phases = mainPhase + vec2(vertexPhase, Color.g) + wPosition.xy * 0.005 ; // + wPosition.xy;
	float trunkPhase = mainPhase - 0.5 * vertexBranchAmplitude;
	
	// Waves
	vec4 waves = (fract(phases.xxyy * vec4(detailFrequencies, branchFrequencies)) * 2.0 - 1.0);
	vec2 waves2 = (fract(trunkPhase * trunkFrequencies) * 2.0 - 1.0);
	
	// + Smoothstep
	//   + Convert from saw to triangle waves
	vec4 triangleWave = abs(waves);
	vec2 triangleWave2 = abs(waves2);
	//   + Cubic hermite
	waves = triangleWave * triangleWave *( 3.0 - 2.0 * triangleWave );  
	waves = waves * 2.0 - 1.0;
	waves2 = triangleWave2 * triangleWave2 *( 3.0 - 2.0 * triangleWave2 );  
	waves2 = waves2 * 2.0 - 1.0;
	
	// + Final waves results
	vec2 wavesSum = waves.xz + waves.yw;

	vec3 displacement = wavesSum.xxy * vec3(detailAmplitude * Normal.xy, branchAmplitude);
	position.xyz += displacement;
	
	vec3 dirTest = windParams2.xyz * displacement.z * windParams2.w;
	position.xyz += dirTest;			
	
	gl_Position = matWorldViewProjection * vec4(position, 1.0);
#else
	gl_Position = matWorldViewProjection * Vertex;
#endif

#ifndef NO_DIFFUSE
    lowp float nDotVP = max(0.0, dot(Normal, Light0DirectionOS)); 
    lowp vec3 Diffuse = Light0DiffuseColor.rgb * nDotVP;
    nDotVP = max(0.0, dot(Normal, Light1DirectionOS)); 
    Diffuse +=  Light1DiffuseColor.rgb * nDotVP;
#else
	lowp vec3 Diffuse = vec3(0.0, 0.0, 0.0);
#endif

	lowp vec3 Ambient = Light0AmbientColor.rgb;
	// Ambient += 0.6 * Light0DiffuseColor.rgb;
	// Diffuse *= 0.5;
 
 	float a = clamp((FadeParams.x - gl_Position.z) * FadeParams.y, 0.0, 1.0);
 	
#ifdef LM
    lowp vec4 color = vec4(1.0, 1.0, 1.0, a);
#else
    lowp vec4 color = vec4(Ambient.xyz + Diffuse.xyz * SunOcclusionFactor, a);
#endif

	vColor = color;
	
	#ifdef DCC_MAX
		if (vColor.rgb == vec3(0.0))
   	 		vColor = vec4(1.0);
    #endif

	vTexCoord0 = texcoord0;

#ifdef UV_SCALE_OFFSET
	vTexCoord0 = vTexCoord0 * texcoord0_scaleoffset.xy + texcoord0_scaleoffset.zw;
#endif

#ifdef LM
	vTexCoord1 = texcoord1;	// LightMap
	
#if defined(UV_SCALE_OFFSET) || defined(UV_SCALE_OFFSET1)
	vTexCoord1 = vTexCoord1 * texcoord1_scaleoffset.xy + texcoord1_scaleoffset.zw;
#endif
#endif // LM

/////////////////////////
//Fog factor output	   //
/////////////////////////
#ifdef FG
	highp float fogDistance = dot(matWorldViewT[2], Vertex);//length(fogVector);
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
} // main end
