#ifdef GL_FRAGMENT_PRECISION_HIGH
	// define maxp based on user preference
	#ifdef GLITCH_USE_HIGHP
		#define MAXP highp
	#else
		#define MAXP mediump
	#endif
#else
	// highp not supported, force maxp as medium
	#define MAXP mediump
#endif

attribute highp vec4 Position;
attribute lowp vec4 Color0;
attribute MAXP vec2 TexCoord0;

uniform MAXP mat4 TexCoord0_transform;
uniform highp mat4 WorldViewProjectionMatrix;

varying MAXP vec2 vTexCoord0;
varying lowp vec4 vColor0;

#ifdef FADE
uniform highp vec2 FadeParams;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

void main(void)
{
    gl_Position = WorldViewProjectionMatrix * Position;
    vTexCoord0 = (TexCoord0_transform * vec4(TexCoord0, 1, 0)).xy;
    vColor0 = Color0;

#ifdef FADE
	float a = clamp((FadeParams.x - gl_Position.z) * FadeParams.y, 0.0, 1.0);
	vColor0.a *= a;
#endif

#ifdef FG
	highp float fogDistance = dot(matWorldViewT[2], Vertex);
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
}
