#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG
//#define SKINNED

attribute highp vec4 Vertex DCC(: POSITION);

attribute mediump vec2 texcoord0 DCC(: TEXCOORD0);
#ifdef UV_SCALE_OFFSET
	uniform mediump vec4 texcoord0_scaleoffset;
#endif

varying mediump vec2 vTexCoord0 DCC(: TEXCOORD0);

#if defined(VERTEX_ALPHA) || defined(VERTEX_COLOR)
	#ifdef DCC_MAX
		attribute lowp    vec3 Color DCC(: TEXCOORD1);
		attribute lowp    float Alpha DCC(: TEXCOORD2);
	#else
		attribute lowp    vec4 Color;
	#endif
#endif

#ifdef UV
	uniform mediump float offsetU;
	uniform mediump float offsetV;
#endif

uniform highp mat4 matWorldViewProjection;

#if defined(OFFSET_VIEW_SPACE_DISTANCE) || defined(OFFSET_VIEW_SPACE_DISTANCE_ANAMORPHIC)
uniform highp vec3 eyePosOS;
#if defined(OFFSET_VIEW_SPACE_DISTANCE)
uniform highp float ViewOffsetDistance;
#endif
#if defined(OFFSET_VIEW_SPACE_DISTANCE_ANAMORPHIC)
uniform highp float ViewOffsetDistanceAnamorphic;
#endif
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

#if defined (AL) || defined (AB)
	uniform lowp float Object_Alpha;
	#ifdef VERTEX_COLOR
		varying lowp vec4 vColorAlpha;
	#else
		varying lowp float vAlpha;
	#endif
#endif

#ifdef GLOBALFADE
	uniform highp float GlobalFade;
#endif

#ifdef FADE
	uniform highp vec2 FadeParams;
#endif

#if defined(SKINNED) && !defined(DCC_MAX)
	#define NUM_BONES 48
	#define NUM_WEIGHT_MASK 1
	uniform highp mat4 BoneMatrices[NUM_BONES];
	uniform highp vec4 WeightMask;
	attribute mediump vec4 SkinWeights;
	attribute mediump vec4 SkinIndices;
#endif

void main()
{	
	// Vertex Position output

#if defined(SKINNED) && !defined(DCC_MAX)
    highp vec4 weights = SkinWeights * WeightMask;
    highp vec4 netPosition = vec4(0.0, 0.0, 0.0, 0.0);

	vec4 indices = SkinIndices;

    for (int i = 0; i < NUM_WEIGHT_MASK; i++)
    {
        int index = int(indices[i]);
        netPosition += weights[i] * (BoneMatrices[index] * vec4(Vertex.xyz, 1));        
    }
  
#else
	highp vec4 netPosition = Vertex;
#endif

#if defined(OFFSET_VIEW_SPACE_DISTANCE) || defined(OFFSET_VIEW_SPACE_DISTANCE_ANAMORPHIC)
	// [ICH] we should parametrize the offset
	highp vec3 diff = Vertex.xyz - eyePosOS;
	highp float diffLen = length(diff);
	#if defined(OFFSET_VIEW_SPACE_DISTANCE)
		highp float customViewOffsetDistance = ViewOffsetDistance;
	#endif
	#if defined(OFFSET_VIEW_SPACE_DISTANCE_ANAMORPHIC)
		highp float customViewOffsetDistance = ViewOffsetDistanceAnamorphic;
	#endif
	highp float finalViewOffsetDistance = min(customViewOffsetDistance, diffLen * 0.85);
	highp vec4 dist = vec4(normalize(diff) * finalViewOffsetDistance, 0.0);
	netPosition = netPosition - dist;
#endif

	gl_Position = matWorldViewProjection * netPosition;

 
	vTexCoord0 = texcoord0;

#if defined(AL) || defined(AB)
	#ifdef VERTEX_COLOR
		vColorAlpha.a = Object_Alpha;
	#else
		vAlpha = Object_Alpha;
	#endif
	#ifdef VERTEX_ALPHA
		#ifdef DCC_MAX
			#ifdef VERTEX_COLOR
				vColorAlpha.a = Alpha;
				vColorAlpha.a += dot(Color, vec3(0.0001));
			#else
				vAlpha *= Alpha;
				vAlpha += dot(Color, vec3(0.0001));
			#endif
		#else
			#ifdef VERTEX_COLOR
				vColorAlpha.rgb = Color.rgb;
				vColorAlpha.a *= Color.a;
			#else
				vAlpha *= Color.a;
			#endif
		#endif
	#endif
#endif

#ifdef UV
	//UV Animations
	vTexCoord0.x += offsetU;
	vTexCoord0.y += offsetV;
#endif

#ifdef GLOBALFADE
	vAlpha *= GlobalFade;
#endif

#ifdef FADE
	vAlpha *= clamp((FadeParams.x - gl_Position.z) * FadeParams.y, 0.0, 1.0);
#endif

#ifdef UV_SCALE_OFFSET
	vTexCoord0 = vTexCoord0 * texcoord0_scaleoffset.xy + texcoord0_scaleoffset.zw;
#endif

/////////////////////////
//Fog factor output	   //
/////////////////////////
#ifdef FG
	highp float fogDistance = dot(matWorldViewT[2], Vertex);//length(fogVector);
	vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
#endif
} // main end
