#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

uniform lowp sampler2D Diffuse;
uniform lowp sampler2D NormalMapSampler_alpha; // Android
uniform lowp sampler2D Diffuse_alpha; // Android

varying lowp vec3 vColor DCC(: COLOR0);

varying highp vec2 vCoord0 DCC(: TEXCOORD0);
varying highp vec2 vCoord1 DCC(: TEXCOORD1);

#ifdef FADE
	varying highp float vZ;
#endif

#if defined (WATER_ALPHA)
	uniform lowp float Water_Alpha;
#endif	

#if defined (MASK) || defined (WATER_DISTORTION)
	uniform lowp sampler2D FrameBuffer;

	#ifdef WATER_DISTORTION
		uniform highp vec3 WaterAverageAmbient;
		uniform highp vec3 WaterSpecColor;
		
		//varying lowp float vAlpha;
		varying highp vec3 vScreenPosition;
		
		uniform lowp sampler2D NormalMapSampler;
		uniform mediump float DistortionFactor;
		
		uniform mediump float FrameBufferScale;

		uniform highp vec3 Light0DirectionOS;
		uniform lowp vec4 Light0Specularcolor;
		uniform highp vec4 Light0Diffusecolor;
		uniform lowp vec4 Light0Ambientcolor;
		
		uniform highp vec3 Light1DirectionOS;
		uniform highp vec4 Light1Diffusecolor;
		
		uniform highp vec4 EyePositionOS;
		
		varying highp vec3 vPosition;
		
		uniform mediump float Glossiness;
		uniform mediump float SpecFactor;
	#endif
#endif

uniform lowp vec4 sunFactor;

#ifdef FADE
	uniform highp vec2 FadeParams;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform lowp vec4 fogcolor;   
#endif

#define TONED 
#ifdef TONED
	uniform lowp vec3 ToneGain;
	uniform lowp vec3 ToneOffset;
#endif

void main()
{
#ifdef MASK
	gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
#else
	#ifndef WATER_DISTORTION
		lowp vec4 color = (texture2D(Diffuse, vCoord0) + texture2D(Diffuse, vCoord1)) * 0.5;
		color.rgb *= vColor;
		#ifdef FADE
			color.a *= clamp((FadeParams.x - vZ) * FadeParams.y, 0.0, 1.0);
		#endif
		
		#ifdef WATER_ALPHA
			color.a *= Water_Alpha;
		#endif		
		
		#ifdef FG
			color.xyz = mix(color.xyz, fogcolor.xyz, vFogFactor);
		#endif // FG	
		
		#ifdef TONED	
			color.rgb = color.rgb * ToneGain + ToneOffset;
		#endif
		
		gl_FragColor = color;
	#else
		highp vec3 NormalMap = texture2D(NormalMapSampler, vCoord0).xyz + texture2D(NormalMapSampler, vCoord1).xyz - 1.0;
		
		highp vec3 ViewDirection = normalize(EyePositionOS.xyz - vPosition.xyz);
		highp vec3 normal = normalize(NormalMap);
		highp float NDotL = dot(normal, Light0DirectionOS);
		highp float specPower = pow(max(0.0, dot(((2.0 * NDotL) * normal) - Light0DirectionOS, ViewDirection)), Glossiness);

	 	highp vec2 baseUV = (0.5 * (vScreenPosition.xy / vScreenPosition.z + 1.0));
	 	baseUV.y = baseUV.y * FrameBufferScale;
	 	highp vec2 distortedUV = baseUV + (NormalMap.xy) * DistortionFactor;
	 		
		lowp vec4 undistorted = texture2D(FrameBuffer, baseUV);
		lowp vec4 distorted = texture2D(FrameBuffer, distortedUV);
	
		lowp vec3 refractionColor = (undistorted.xyz * distorted.a + distorted.xyz * (1.0 - distorted.a));
		
		lowp vec4 color;
		
		color.rgb = refractionColor;
		
		highp float NDotL2 = dot(normal, Light1DirectionOS);
		//average the ambient, so that it's on the gray scale.
		color.rgb *= WaterAverageAmbient + Light0Diffusecolor.rgb * abs(NDotL) + Light1Diffusecolor.rgb * abs(NDotL2);
		//only apply the tone mapping on the specular light, everything under water is already tone mapped...
		//UPDATE: actually... no tone mapping at all works better...
		//#ifdef TONED	
		//	color.rgb += WaterSpecColor * specPower * ToneGain + ToneOffset;
		//#else
			color.rgb += WaterSpecColor * specPower;
		//#endif
		
		color.a = 1.0;

		//#ifdef TONED	
		//	color.rgb = color.rgb * ToneGain + ToneOffset;
		//#endif

		#ifdef FADE
			lowp float a = /*vAlpha **/ clamp((FadeParams.x - vZ) * FadeParams.y, 0.0, 1.0);
		#else
			lowp float a = 1.0;//vAlpha;
		#endif
		
		//"vertex alpha" by interpolating between undistorted frame buffer and water.
		lowp vec4 finalColor = vec4(mix(undistorted.rgb, color.rgb, a), 0.0);
		
		#ifdef FG
			finalColor.xyz = mix(finalColor.xyz, fogcolor.xyz, vFogFactor);
		#endif // FG
		
		gl_FragColor = finalColor;
	#endif
#endif


}
