#ifdef DCC_MAX
	#define DCC(x) x
#else
	#define DCC(x)
#endif

//#define FG

attribute highp   vec3 Normal DCC(: NORMAL);
attribute highp   vec4 Vertex DCC(: POSITION);
attribute highp	  vec2 TexCoord0 DCC(: TEXCOORD0);
#ifdef DCC_MAX
	attribute lowp    vec3 Color DCC(: TEXCOORD1);
	attribute lowp    float Alpha DCC(: TEXCOORD2);
#else
	attribute lowp    vec4 Color;
#endif

uniform highp float Time;

uniform   highp   mat4 WorldViewProjectionMatrix;
uniform   highp   mat4 World;
uniform   highp   mat4 WorldIT;

varying	  highp vec2 vCoord0 DCC(: TEXCOORD0);
varying	  highp vec2 vCoord1 DCC(: TEXCOORD1);
varying	  lowp  vec3 vColor DCC(: COLOR0);

uniform mediump float WaveUSpeed;
uniform mediump float WaveVSpeed;
uniform mediump float WaterNormalTiling;

uniform mediump float WaveUSpeed2;
uniform mediump float WaveVSpeed2;
uniform mediump float WaterNormalTiling2;

#ifdef WATER_DISTORTION
	varying highp vec3 vScreenPosition;
	varying highp vec3 vPosition;
	//varying lowp float vAlpha;
#else
	uniform highp vec4 Light0Diffusecolor;
	uniform highp vec4 Light1Diffusecolor;
#endif

#ifdef FADE
	varying highp float vZ;
#endif

#ifdef FG
	varying lowp float vFogFactor;
	uniform highp vec2 fogstartend;
	uniform highp mat4 matWorldViewT;
#endif

void main(void)
{
	gl_Position = WorldViewProjectionMatrix * Vertex;

#if !defined (MASK)
	vCoord0 = TexCoord0;
	
	#ifdef WATER_DISTORTION
	/*	#ifdef DCC_MAX
			vAlpha = Alpha;
		#else
			vAlpha = Color.a;
		#endif*/
	#else
		vec3 light = Light0Diffusecolor.rgb + Light1Diffusecolor.rgb;
		#ifdef DCC_MAX
			if (light.rgb == vec3(0.0))
				light = vec3(1.0);
		#endif
		vColor = vec3(Color.rgb * light);
	#endif
	
	#if defined(WATER_DISTORTION)
		vScreenPosition.xy = gl_Position.xy;
		vScreenPosition.z = gl_Position.w;
		
		vPosition = Vertex.xyz;
	#endif

	#ifdef FADE
		vZ = gl_Position.z;
	#endif
	
	vCoord0 = TexCoord0 * WaterNormalTiling + Time * vec2(WaveUSpeed, WaveVSpeed);
	vCoord1 = TexCoord0 * WaterNormalTiling2 + Time * vec2(WaveUSpeed2, WaveVSpeed2);
	
	#ifdef FG
		highp float fogDistance = dot(matWorldViewT[2], Vertex);
		vFogFactor = clamp((-fogDistance - fogstartend[0]) / (fogstartend[1] - fogstartend[0]), 0.0, 1.0);
	#endif
	
#endif
}
